package com.demo.wjj.controller;

import com.demo.wjj.converter.UserRegisterConverter;
import com.demo.wjj.qo.UserRegisterQo;
import com.demo.wjj.service.SaleService;
import com.demo.wjj.service.UserService;
import com.demo.wjj.utils.ApiResult;
import com.demo.wjj.utils.EmojiFilterUtil;
import com.demo.wjj.utils.ExecuteResult;
import com.demo.wjj.utils.Result;
import com.demo.wjj.validator.UserRegisterQoValidator;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.BindingResult;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 用户
 * @author wangqing
 * @date 2018.11.10
 */
@RestController
@RequestMapping("/user")
public class UserController {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    @Autowired
    private UserService userService;

    @Autowired
    private UserRegisterQoValidator userRegisterQoValidator;
    @Autowired
    private UserRegisterConverter userRegisterConverter;
    @Autowired
    private SaleService saleService;//icegan--新增

    /**
     * 精品机账号
     */
    @Value("ZHXTCS")
    private String excellentAccount;

    @InitBinder
    protected void initBinder(WebDataBinder binder) {
        Object target = binder.getTarget();
        if (target == null) {
            return;
        }
        if (userRegisterQoValidator.supports(target.getClass())) {
            binder.addValidators(userRegisterQoValidator);
        }
    }

    /**
     * 用户注册
     * @param userRegisterQo 注册参数
     * @param bindingResult 绑定结果
     * @return apiResult
     */
    @PostMapping("/register")
    public ApiResult register(@Valid UserRegisterQo userRegisterQo, BindingResult bindingResult) {
        LOG.info("调用用户注册(/user/register)接口, userRegisterQo:{}", userRegisterQo);
        if(excellentAccount.equals(userRegisterQo.getAgentId())){
            userRegisterQo.setRegistType("2");
        }
        String nickname = userRegisterQo.getNickname();
        String filterName = EmojiFilterUtil.filterEmoji(nickname);
        String regEx="[`~!@#$%^&*()+=|{}':;',\\[\\].<>/?~！@#￥%……&*（）——+|{}【】‘；：”“’。，、？]";
        Pattern p = Pattern.compile(regEx);
        Matcher m = p.matcher(filterName);
        String trim = m.replaceAll("0").trim();
        userRegisterQo.setNickname(trim);

//        List<ObjectError> errors = bindingResult.getAllErrors();
//        if (CollectionUtils.isNotEmpty(errors)) {
//            LOG.info("调用用户注册(/user/register)接口, 请求参数错误");
//            return ApiResult.createFailure();
//        }

        try {
            ApiResult<Boolean> apiResult;
            ExecuteResult<Boolean> executeResult = userService.register(userRegisterConverter.convert(userRegisterQo));
            if (!executeResult.isExecuteResult()) {
                Result result = executeResult.getResult();
                if (result != null) {
                    apiResult = new ApiResult<>(result);
                } else {
                    apiResult = ApiResult.createFailure();
                }
            } else {
                apiResult = executeResult.getData() ? ApiResult.createSuccess(null) : ApiResult.createFailure();
            }
            LOG.info("调用用户注册(/user/register)接口成功, apiResult => {}", apiResult);

            //icegan--新增，认证成功则增加销售用户1000积分
            if(apiResult.getCode().equals(Result.SUCCESS.getCode())){
                LOG.info("实名认证成功，为销售员，增加1000积分");
                saleService.addIntegral(userRegisterQo.getOpenId(),1000);
            }
            return apiResult;
        } catch (Exception e) {
            LOG.error("调用用户注册(/user/register)接口异常, userRegisterQo:{}", userRegisterQo, e);
            return ApiResult.createFailure();
        }
    }

    /**
     * 查询微信号是否已注册销售员
     * @param agentId 商家id
     * @param openId 微信openId
     * @return apiResult
     */
    @GetMapping("/querySale")
    public ApiResult querySale(String agentId, String openId) {
        LOG.info("调用查询微信号是否已注册销售员(/user/querySale)接口, agentId:{}, openId:{}", agentId, openId);
        try {
            boolean exists = userService.querySale(agentId, openId);
            Map<String, Integer> result = new HashMap<>();
            result.put("register", exists ? 1 : 0);
            ApiResult<Map<String, Integer>> apiResult = ApiResult.createSuccess(result);
            LOG.info("调用查询微信号是否已注册销售员(/user/querySale)接口成功, apiResult:{}", apiResult);
            return apiResult;
        } catch (Exception e) {
            LOG.error("调用查询微信号是否已注册销售员(/user/querySale)接口异常, agentId:{}, openId:{}", agentId, openId, e);
            return ApiResult.createFailure();
        }
    }

    /**
     * 查询微信号是否已注册车商
     * @param agentId 商家id
     * @param openId 微信openId
     * @return apiResult
     */
    @GetMapping("/queryDiggerAgent")
    public ApiResult queryDiggerAgent(String agentId, String openId) {
        LOG.info("调用查询微信号是否已注册车商(/user/queryDiggerAgent)接口, agentId:{}, openId:{}", agentId, openId);
        try {
            boolean exists = userService.queryDiggerAgent(agentId, openId);
            Map<String, Integer> result = new HashMap<>();
            result.put("register", exists ? 1 : 0);
            ApiResult<Map<String, Integer>> apiResult = ApiResult.createSuccess(result);
            LOG.info("调用查询微信号是否已注册车商(/user/queryDiggerAgent)接口成功, apiResult:{}", apiResult);
            return apiResult;
        } catch (Exception e) {
            LOG.error("调用查询微信号是否已注册车商(/user/queryDiggerAgent)接口异常, agentId:{}, openId:{}", agentId, openId, e);
            return ApiResult.createFailure();
        }
    }

    /**
     * 解绑
     * @param agentId 商家id
     * @param openId 微信openId
     * @param unbindFlag 解绑标识 1:解绑销售员; 2: 解绑车商; 3:解绑销售员和车商
     * @return
     */
    @PostMapping("/unbind")
    public ApiResult unbind(@RequestParam(required = false) String agentId,
                            @RequestParam(required = false) String openId,
                            @RequestParam(required = false) String unbindFlag) {
        LOG.info("调用解绑用户(/user/unbind)接口, agentId:{}, openId:{}, unbindFlag:{}", agentId, openId, unbindFlag);
        if (StringUtils.isBlank(agentId)) {
            LOG.info("agentId为空");
            return ApiResult.createFailure();
        }

        if (StringUtils.isBlank(openId)) {
            LOG.info("openId为空");
            return ApiResult.createFailure();
        }

        final String[] unbindFlags = {"1", "2", "3"};
        if (StringUtils.isBlank(unbindFlag) || !ArrayUtils.contains(unbindFlags, unbindFlag)) {
            LOG.info("unbindFlag值非法, unbindFloag:{}", unbindFlag);
            return ApiResult.createFailure();
        }

        try {
            boolean unbindResult = userService.unbindUser(agentId, openId, unbindFlag);
            ApiResult apiResult = unbindResult ? ApiResult.createSuccess(null) : ApiResult.createFailure();
            LOG.info("调用绑定用户(/user/unbind)接口成功, apiResult:{}", apiResult);
            return apiResult;
        } catch (Exception e) {
            LOG.error("调用解绑用户(/user/unbind)接口异常", e);
            return ApiResult.createFailure();
        }
    }
}






//package com.demo.wjj.controller;
//
//import com.demo.wjj.converter.UserRegisterConverter;
//import com.demo.wjj.qo.UserRegisterQo;
//import com.demo.wjj.service.UserService;
//import com.demo.wjj.utils.ApiResult;
//import com.demo.wjj.utils.ExecuteResult;
//import com.demo.wjj.utils.Result;
//import com.demo.wjj.validator.UserRegisterQoValidator;
//import org.apache.commons.collections4.CollectionUtils;
//import org.apache.commons.lang3.ArrayUtils;
//import org.apache.commons.lang3.StringUtils;
//import org.slf4j.Logger;
//import org.slf4j.LoggerFactory;
//import org.springframework.beans.factory.annotation.Autowired;
//import org.springframework.validation.BindingResult;
//import org.springframework.validation.ObjectError;
//import org.springframework.web.bind.WebDataBinder;
//import org.springframework.web.bind.annotation.*;
//
//import javax.validation.Valid;
//import java.util.HashMap;
//import java.util.List;
//import java.util.Map;
//
///**
// * 用户
// * @author wangqing
// * @date 2018.11.10
// */
//@RestController
//@RequestMapping("/user")
//public class UserController {
//
//    private final Logger LOG = LoggerFactory.getLogger(getClass());
//
//    @Autowired
//    private UserService userService;
//
//    @Autowired
//    private UserRegisterQoValidator userRegisterQoValidator;
//    @Autowired
//    private UserRegisterConverter userRegisterConverter;
//
//    @InitBinder
//    protected void initBinder(WebDataBinder binder) {
//        Object target = binder.getTarget();
//        if (target == null) {
//            return;
//        }
//        if (userRegisterQoValidator.supports(target.getClass())) {
//            binder.addValidators(userRegisterQoValidator);
//        }
//    }
//
//    /**
//     * 用户注册
//     * @param userRegisterQo 注册参数
//     * @param bindingResult 绑定结果
//     * @return apiResult
//     */
//    @PostMapping("/register")
//    public ApiResult register(@Valid UserRegisterQo userRegisterQo, BindingResult bindingResult) {
//        LOG.info("调用用户注册(/user/register)接口, userRegisterQo:{}", userRegisterQo);
//
//        List<ObjectError> errors = bindingResult.getAllErrors();
//        if (CollectionUtils.isNotEmpty(errors)) {
//            LOG.info("调用用户注册(/user/register)接口, 请求参数错误");
//            return ApiResult.createFailure();
//        }
//
//        try {
//            ApiResult<Boolean> apiResult;
//            ExecuteResult<Boolean> executeResult = userService.register(userRegisterConverter.convert(userRegisterQo));
//            if (!executeResult.isExecuteResult()) {
//                Result result = executeResult.getResult();
//                if (result != null) {
//                    apiResult = new ApiResult<>(result);
//                } else {
//                    apiResult = ApiResult.createFailure();
//                }
//            } else {
//                apiResult = executeResult.getData() ? ApiResult.createSuccess(null) : ApiResult.createFailure();
//            }
//            LOG.info("调用用户注册(/user/register)接口成功, apiResult => {}", apiResult);
//            return apiResult;
//        } catch (Exception e) {
//            LOG.error("调用用户注册(/user/register)接口异常, userRegisterQo:{}", userRegisterQo, e);
//            return ApiResult.createFailure();
//        }
//    }
//
//    /**
//     * 查询微信号是否已注册销售员
//     * @param agentId 商家id
//     * @param openId 微信openId
//     * @return apiResult
//     */
//    @GetMapping("/querySale")
//    public ApiResult querySale(String agentId, String openId) {
//        LOG.info("调用查询微信号是否已注册销售员(/user/querySale)接口, agentId:{}, openId:{}", agentId, openId);
//        try {
//            boolean exists = userService.querySale(agentId, openId);
//            Map<String, Integer> result = new HashMap<>();
//            result.put("register", exists ? 1 : 0);
//            ApiResult<Map<String, Integer>> apiResult = ApiResult.createSuccess(result);
//            LOG.info("调用查询微信号是否已注册销售员(/user/querySale)接口成功, apiResult:{}", apiResult);
//            return apiResult;
//        } catch (Exception e) {
//            LOG.error("调用查询微信号是否已注册销售员(/user/querySale)接口异常, agentId:{}, openId:{}", agentId, openId, e);
//            return ApiResult.createFailure();
//        }
//    }
//
//    /**
//     * 查询微信号是否已注册车商
//     * @param agentId 商家id
//     * @param openId 微信openId
//     * @return apiResult
//     */
//    @GetMapping("/queryDiggerAgent")
//    public ApiResult queryDiggerAgent(String agentId, String openId) {
//        LOG.info("调用查询微信号是否已注册车商(/user/queryDiggerAgent)接口, agentId:{}, openId:{}", agentId, openId);
//        try {
//            boolean exists = userService.queryDiggerAgent(agentId, openId);
//            Map<String, Integer> result = new HashMap<>();
//            result.put("register", exists ? 1 : 0);
//            ApiResult<Map<String, Integer>> apiResult = ApiResult.createSuccess(result);
//            LOG.info("调用查询微信号是否已注册车商(/user/queryDiggerAgent)接口成功, apiResult:{}", apiResult);
//            return apiResult;
//        } catch (Exception e) {
//            LOG.error("调用查询微信号是否已注册车商(/user/queryDiggerAgent)接口异常, agentId:{}, openId:{}", agentId, openId, e);
//            return ApiResult.createFailure();
//        }
//    }
//
//    /**
//     * 解绑
//     * @param agentId 商家id
//     * @param openId 微信openId
//     * @param unbindFlag 解绑标识 1:解绑销售员; 2: 解绑车商; 3:解绑销售员和车商
//     * @return
//     */
//    @PostMapping("/unbind")
//    public ApiResult unbind(@RequestParam(required = false) String agentId,
//                            @RequestParam(required = false) String openId,
//                            @RequestParam(required = false) String unbindFlag) {
//        LOG.info("调用解绑用户(/user/unbind)接口, agentId:{}, openId:{}, unbindFlag:{}", agentId, openId, unbindFlag);
//        if (StringUtils.isBlank(agentId)) {
//            LOG.info("agentId为空");
//            return ApiResult.createFailure();
//        }
//
//        if (StringUtils.isBlank(openId)) {
//            LOG.info("openId为空");
//            return ApiResult.createFailure();
//        }
//
//        final String[] unbindFlags = {"1", "2", "3"};
//        if (StringUtils.isBlank(unbindFlag) || !ArrayUtils.contains(unbindFlags, unbindFlag)) {
//            LOG.info("unbindFlag值非法, unbindFloag:{}", unbindFlag);
//            return ApiResult.createFailure();
//        }
//
//        try {
//            boolean unbindResult = userService.unbindUser(agentId, openId, unbindFlag);
//            ApiResult apiResult = unbindResult ? ApiResult.createSuccess(null) : ApiResult.createFailure();
//            LOG.info("调用绑定用户(/user/unbind)接口成功, apiResult:{}", apiResult);
//            return apiResult;
//        } catch (Exception e) {
//            LOG.error("调用解绑用户(/user/unbind)接口异常", e);
//            return ApiResult.createFailure();
//        }
//    }
//}
