package com.demo.wjj.service.impl;

import com.demo.wjj.bo.SaveDisplaceBo;
import com.demo.wjj.mapper.DisplaceManageMapper;
import com.demo.wjj.po.Agent;
import com.demo.wjj.po.AttrContent;
import com.demo.wjj.po.DisplaceAudit;
import com.demo.wjj.po.Sale;
import com.demo.wjj.service.*;
import com.demo.wjj.utils.CommitTransactional;
import com.demo.wjj.utils.ExecuteResult;
import com.demo.wjj.utils.Result;
import com.demo.wjj.utils.UuidUtils;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanWrapper;
import org.springframework.beans.BeanWrapperImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 置换管理
 * @author wangqing
 * @date 2018.11.12
 */
@Service
public class DisplaceManageImpl implements DisplaceManage {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    private static final Map<String, String> attrContentKey = new HashMap<>();

    @Autowired
    private SaleService saleService;

    @Autowired
    private AgentService agentService;

    @Autowired
    private DisplaceAuditService displaceAuditService;

    @Autowired
    private AttrContentService attrContentService;

    @Autowired
    private RegionService regionService;

    @Autowired
    private DictionaryService dictionaryService;

    @Autowired
    private WeiXinMessageService weiXinMessageService;

    @Autowired
    private DisplaceManageMapper displaceManageMapper;

    static {
        // 品牌
        attrContentKey.put("brand", "sbpp");
        // 型号
        attrContentKey.put("model", "sbxh");
        // 设备类型
        attrContentKey.put("deviceType", "sblx");
        // 出厂年份
        attrContentKey.put("yearMonth", "sbccnf");
        // 使用小时
        attrContentKey.put("hours", "sbsyxs");
        // 所在地
        attrContentKey.put("location", "sbszd");
        // 配置说明
        attrContentKey.put("config", "svpzsm");
        // 作业履历
        attrContentKey.put("jobResume", "sbzyll");
        // 票证说明
        attrContentKey.put("ticket", "sbpzsm");
    }

    @CommitTransactional
    @Override
    public ExecuteResult<Boolean> save(SaveDisplaceBo saveDisplaceBo) {
        LOG.info("进入保存置换, saveDisplaceBo:{}", saveDisplaceBo);

        boolean existsSale = saleService.existsOpenId(saveDisplaceBo.getAgentId(), saveDisplaceBo.getOpenId());
        if (!existsSale) {
            LOG.warn("微信用户未注册销售员, agentId:{}, openId:{}", saveDisplaceBo.getAgentId(), saveDisplaceBo.getOpenId());
            return new ExecuteResult<Boolean>()
                    .setExecuteResult(false)
                    .setResult(Result.NO_AUTHORITY);
        }

        final String agentId = saveDisplaceBo.getAgentId();
        Agent agent = agentService.getAgent(agentId);
        if (agent == null) {
            LOG.warn("未查找到商家信息, agentId:{}", saveDisplaceBo.getAgentId());
            throw new RuntimeException("未查找到商家信息, agentId:{}" + saveDisplaceBo.getAgentId());
        }

        final String openId = saveDisplaceBo.getOpenId();
        Sale sale = saleService.getSale(agentId, openId);
        if (sale == null) {
            LOG.warn("未查找到销售人员, agentId:{}, openId:{}", agent, openId);
            throw new RuntimeException("未查找到销售员, agentId:" + agentId + ", openId:" + openId);
        }

        //置换id是否为空  为空时返回false  有返回true
        boolean updateOperate = StringUtils.isNoneBlank(saveDisplaceBo.getDisplaceId());
        DisplaceAudit displaceAudit = createDisplaceAudit(saveDisplaceBo, agent, sale);
        boolean saveDisplace = displaceAuditService.save(displaceAudit, updateOperate);
        LOG.debug("保存置换审核, result => {}", saveDisplace);
        if (!saveDisplace) {
            LOG.info("保存置换审核失败");
            return new ExecuteResult<Boolean>()
                    .setExecuteResult(true)
                    .setData(false);
        }

        if (updateOperate) {
            LOG.info("删除属性内容, displaceId:{}", displaceAudit.getId());
            attrContentService.deleteAttrContents(displaceAudit.getId());
        }

        saveDisplaceBo.setLocation(regionService.assemblyArea(saveDisplaceBo.getProvinceId(), saveDisplaceBo.getCityId(), saveDisplaceBo.getCountyId())[1]);

        List<AttrContent> attrContents = createAttrContents(displaceAudit.getDisplaceId(), saveDisplaceBo);
        boolean saveAttrContent = attrContentService.save(attrContents);
        LOG.debug("保存属性内容, result => {}", saveAttrContent);
        if (!saveAttrContent) {
            LOG.info("保存属性内容失败");
            throw new RuntimeException("保存属性内容失败");
        }

        saveDisplaceBo.setDisplaceId(displaceAudit.getDisplaceId());
        boolean sendResult=true;
        //暂存状态不推送消息
        if("1".equals(saveDisplaceBo.getStorage())){//1-完成；2-暂存
            sendResult = weiXinMessageService.send(agent, sale, saveDisplaceBo);
        }

        LOG.info("微信推送置换审核信息, sendResult => {}", sendResult);

        LOG.info("退出保存置换");
        return new ExecuteResult<Boolean>().setExecuteResult(true).setData(true);
    }

    /**
     * 创建属性内容
     * @param displaceAuditId 置换审核id
     * @param saveDisplaceBo 保存置换
     * @return 属性内容
     */
    private List<AttrContent> createAttrContents(String displaceAuditId, SaveDisplaceBo saveDisplaceBo) {
        LOG.info("进入创建属性内容, saveDisplaceBo:{}", saveDisplaceBo);

        Map<String, String> dictionaryMap = dictionaryService.queryDictionaryMap();

        // 将数据字典值转成名称
        saveDisplaceBo.setDeviceType(dictionaryMap.get(saveDisplaceBo.getDeviceType()));
        saveDisplaceBo.setConfig(dictionaryMap.get(saveDisplaceBo.getConfig()));
        saveDisplaceBo.setJobResume(dictionaryMap.get(saveDisplaceBo.getJobResume()));

        List<AttrContent> attrContents = new ArrayList<>();
        BeanWrapper bean = new BeanWrapperImpl(saveDisplaceBo);
        Iterator<String> iterator = attrContentKey.keySet().iterator();
        String attribute;
        AttrContent attrContent;
        // 属性
        while (iterator.hasNext()) {
            attribute = iterator.next();

            attrContent = new AttrContent();
            attrContent.setId(UuidUtils.generate());
            attrContent.setAttrCode(attrContentKey.get(attribute));
            attrContent.setAttrContent((String) bean.getPropertyValue(attribute));
            attrContent.setReferId(displaceAuditId);
            attrContent.setState("1");
            attrContent.setAttrType(AttrContent.ATTR_TYPE_DEVICE);
            attrContents.add(attrContent);
        }

        // 图片
        List<SaveDisplaceBo.DisplaceImageList> pictureList = saveDisplaceBo.getPicture();
        for (SaveDisplaceBo.DisplaceImageList picture : pictureList) {
            final List<String> picList = picture.getPic();
            if (CollectionUtils.isEmpty(picList)) {
                LOG.info("设备图片为空, attr_code:{}", picture.getCode());
                continue;
            }
            int index = 1;
            for (String pic : picList) {
                attrContent = new AttrContent();
                attrContent.setId(UuidUtils.generate());
                attrContent.setAttrCode(picture.getCode());
                attrContent.setAttrContent(pic);
                //保存细节id
                attrContent.setDetailId(picture.getDetailId());
                attrContent.setReferId(displaceAuditId);
                attrContent.setState("1");
                attrContent.setAttrType(AttrContent.ATTR_TYPE_IMAGE);
                attrContent.setSortNo(index++);
                attrContents.add(attrContent);
            }
        }

        LOG.info("退出创建属性内容");
        LOG.debug("创建属性内容, attrContents:{}", attrContents);
        return attrContents;
    }

    /**
     * 创建置换审核
     * @param saveDisplaceBo 保存置换
     * @param agent
     * @param sale
     * @return 置换审核实体
     */
    private DisplaceAudit createDisplaceAudit(SaveDisplaceBo saveDisplaceBo, Agent agent, Sale sale) {
        LOG.info("进入创建置换审核, saveDisplaceBo:{}", saveDisplaceBo);

        DisplaceAudit displaceAudit = new DisplaceAudit();
        if (StringUtils.isBlank(saveDisplaceBo.getDisplaceId())) {
            displaceAudit.setId(UuidUtils.generate());
        } else {
            displaceAudit.setId(saveDisplaceBo.getDisplaceId());
            displaceAudit.setDisplaceId(displaceManageMapper.selectDisplaceId(saveDisplaceBo.getDisplaceId()));
        }
        displaceAudit.setAgentId(agent.getAgentId());
        displaceAudit.setAgentName(agent.getAgentName());
        displaceAudit.setIdeaPrice(saveDisplaceBo.getPrice());
        displaceAudit.setJoumalistName(sale.getSaleName());
        displaceAudit.setJoumalistTel(sale.getSalePhone());
        displaceAudit.setAuditStatus(DisplaceAudit.AUDIT_STATUS_UNDO);
        displaceAudit.setState(DisplaceAudit.STATE_SHOW);
        displaceAudit.setSaleId(sale.getSaleId());
        displaceAudit.setOthersDesc(saveDisplaceBo.getOther());
        displaceAudit.setPubTime(new Date());
        displaceAudit.setIs_good(saveDisplaceBo.getIs_good());
        displaceAudit.setStorage(saveDisplaceBo.getStorage());

        final List<SaveDisplaceBo.DisplaceImageList> picture = saveDisplaceBo.getPicture();
        if (CollectionUtils.isNotEmpty(picture)) {
            List<String> pic;
            final String[] pictureSubfixs = {".BMP", ".JPG", ".JPEG", ".PNG", ".GIF", ".bmp", ".jpg", ".jpeg", ".png", ".gif"};

            SaveDisplaceBo.DisplaceImageList displaceImageList;
            boolean hasFirstPicture = false;
            for (int i = 0, step = picture.size(); i < step && !hasFirstPicture; i++) {
                displaceImageList = picture.get(i);

                pic = displaceImageList.getPic();
                if (CollectionUtils.isNotEmpty(pic)) {
                    String p;
                    for (int j = 0, ostep = pic.size(); j < ostep; j++) {
                        p  = pic.get(j);
                        if (StringUtils.isNoneBlank(p) && p.indexOf(".") != -1) {
                            String subfix = p.substring(p.lastIndexOf("."));
                            if (ArrayUtils.contains(pictureSubfixs, subfix)) {
                                //取第一张图片
                                displaceAudit.setFirstPicUrl(p);
                                hasFirstPicture = true;
                                break;
                            }
                        }
                    }
                }
            }

        }

        LOG.info("退出创建置换审核, displaceAudit:{}", displaceAudit);
        return displaceAudit;
    }
}
