package com.demo.wjj.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.demo.wjj.bo.WeiXinUserBo;
import com.demo.wjj.po.Agent;
import com.demo.wjj.service.AgentService;
import com.demo.wjj.service.OpenIdService;
import com.demo.wjj.utils.ExecuteResult;
import com.demo.wjj.utils.HttpResult;
import com.demo.wjj.utils.HttpUtils;
import com.demo.wjj.utils.Result;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.HashMap;

/**
 * 微信openId
 * @author wangqing
 * @date 2018.11.16
 */
@Service
public class OpenIdServiceImpl implements OpenIdService {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    @Autowired
    private AgentService agentService;

    @Value("${platform.appId}")
    private String platformAppId;

    @Override
    public ExecuteResult<WeiXinUserBo> getOpenId(String agentId, String code) {
        LOG.info("进入获取微信openId, agentId:{}, code:{}", agentId, code);

        if (StringUtils.isBlank(agentId)) {
            LOG.info("agentId为空");
            return new ExecuteResult<WeiXinUserBo>()
                    .setExecuteResult(false)
                    .setResult(Result.Failure);
        }

        if (StringUtils.isBlank(code)) {
            LOG.info("code为空");
            return new ExecuteResult<WeiXinUserBo>()
                    .setExecuteResult(false)
                    .setResult(Result.Failure);
        }

        Agent agent = agentService.getAgent(agentId);
        if (agent == null) {
            LOG.info("未查找到商家信息, agentId:{}", agentId);
            return new ExecuteResult<WeiXinUserBo>()
                    .setExecuteResult(false)
                    .setResult(Result.Failure);
        }

        ExecuteResult<WeiXinUserBo> getOpenIdResult = internalOpenId(code, agent.getAppId(), agent.getAppSecret());
        if (!getOpenIdResult.isExecuteResult()) {
            LOG.info("获取微信openId失败");
            return getOpenIdResult;
        }

        WeiXinUserBo data = getOpenIdResult.getData();

        ExecuteResult<WeiXinUserBo> getNicknameResult = getNickname(data.getAccessToken(), data.getOpenId());
        if (!getNicknameResult.isExecuteResult()) {
            LOG.info("获取用户信息失败");
            return getNicknameResult;
        }

        WeiXinUserBo result = new WeiXinUserBo();
        result.setOpenId(data.getOpenId());
        result.setNickname(getNicknameResult.getData().getNickname());
        LOG.info("退出获取用户openId, result => {}", result);

        return new ExecuteResult<WeiXinUserBo>()
                .setExecuteResult(true)
                .setData(result);

    }

    private ExecuteResult<WeiXinUserBo> internalOpenId(String code, String appId, String appSecret) {
        if (StringUtils.isBlank(appId)) {
            LOG.info("商家appId为空");
            return new ExecuteResult<WeiXinUserBo>()
                    .setExecuteResult(false)
                    .setResult(Result.Failure);
        }

        if (StringUtils.isBlank(appSecret)) {
            LOG.info("商家appSecret为空");
            return new ExecuteResult<WeiXinUserBo>()
                    .setExecuteResult(false)
                    .setResult(Result.Failure);
        }

//        final String apiUrlStr = "https://api.weixin.qq.com/sns/oauth2/access_token?appid=%s&secret=%s&code=%s&grant_type=authorization_code";
//        String apiUrl = String.format(apiUrlStr, appId, appSecret, code);

        final WeChatServiceImpl.Token token = WeChatServiceImpl.TOKEN_CACHE.get(platformAppId);
        if (token == null) {
            LOG.info("未获取到第三方平台token");
            return new ExecuteResult<WeiXinUserBo>()
                    .setExecuteResult(false)
                    .setResult(Result.Failure);
        }

        final String apiUrlStr = "https://api.weixin.qq.com/sns/oauth2/component/access_token?appid=%s&code=%s&grant_type=authorization_code&component_appid=%s&component_access_token=%s";
        String apiUrl = String.format(apiUrlStr, appId, code, platformAppId, token.getContent());

        HttpResult httpResult = HttpUtils.post(apiUrl, new HashMap<String, Object>());

        if (httpResult.getStatusCode() != 200) {
            LOG.info("请求微信获取openId接口失败");
            return new ExecuteResult<WeiXinUserBo>()
                    .setExecuteResult(false)
                    .setResult(Result.Failure);
        }

        String content = httpResult.getContent();
        if (StringUtils.isBlank(content)) {
            LOG.info("请求微信获取openId接口，响应内容为空");
            return new ExecuteResult<WeiXinUserBo>()
                    .setExecuteResult(false)
                    .setResult(Result.Failure);
        }

        try {
            JSONObject json = JSON.parseObject(content);
            String openid = json.getString("openid");
            String accessToken = json.getString("access_token");

            WeiXinUserBo weiXinUserBo = new WeiXinUserBo();
            weiXinUserBo.setOpenId(openid);
            weiXinUserBo.setAccessToken(accessToken);

            LOG.info("退出获取微信openId, openId:{}", openid);
            return new ExecuteResult<WeiXinUserBo>()
                    .setExecuteResult(true)
                    .setData(weiXinUserBo);
        } catch (Exception e) {
            return new ExecuteResult<WeiXinUserBo>()
                    .setExecuteResult(false)
                    .setResult(Result.Failure);
        }
    }

    /**
     * 获取微信用户昵称
     * @param accessToken accessToken
     * @param openId openId
     * @return 微信用户昵称
     */
    private ExecuteResult<WeiXinUserBo> getNickname(String accessToken, String openId) {
        LOG.info("进入获取微信用户昵称, accessToken:{}, openId:{}", accessToken, openId);

        if (StringUtils.isBlank(accessToken)) {
            LOG.info("accessToken为空");
            return new ExecuteResult<WeiXinUserBo>()
                    .setExecuteResult(false)
                    .setResult(Result.Failure);
        }

        if (StringUtils.isBlank(openId)) {
            LOG.info("openId为空");
            return new ExecuteResult<WeiXinUserBo>()
                    .setExecuteResult(false)
                    .setResult(Result.Failure);
        }

        final String apiUrlStr = "https://api.weixin.qq.com/sns/userinfo?access_token=%s&openid=%s&lang=zh_CN";
        String apiUrl = String.format(apiUrlStr, accessToken, openId);

        HttpResult httpResult = HttpUtils.post(apiUrl, new HashMap<String, Object>());

        if (httpResult.getStatusCode() != 200) {
            LOG.info("请求微信获取用户信息失败");
            return new ExecuteResult<WeiXinUserBo>()
                    .setExecuteResult(false)
                    .setResult(Result.Failure);
        }

        String content = httpResult.getContent();
        if (StringUtils.isBlank(content)) {
            LOG.info("请求微信获取用户信息失败，响应内容为空");
            return new ExecuteResult<WeiXinUserBo>()
                    .setExecuteResult(false)
                    .setResult(Result.Failure);
        }

        try {
            JSONObject json = JSON.parseObject(content);
            String nickname = json.getString("nickname");

            WeiXinUserBo weiXinUserBo = new WeiXinUserBo();
            weiXinUserBo.setNickname(nickname);

            LOG.info("退出微信获取用户信息昵称, nickname:{}, openId:{}", nickname, openId);
            return new ExecuteResult<WeiXinUserBo>()
                    .setExecuteResult(true)
                    .setData(weiXinUserBo);
        } catch (Exception e) {
            return new ExecuteResult<WeiXinUserBo>()
                    .setExecuteResult(false)
                    .setResult(Result.Failure);
        }
    }

    @Override
    public ExecuteResult<WeiXinUserBo> getPayOpenId(String agentId, String code) {
        LOG.info("进入获取商家支付openid, agentId:{}, code:{}");

        if (StringUtils.isBlank(agentId)) {
            LOG.info("agentId为空");
            return new ExecuteResult<WeiXinUserBo>()
                    .setExecuteResult(false)
                    .setResult(Result.Failure);
        }

        if (StringUtils.isBlank(code)) {
            LOG.info("code为空");
            return new ExecuteResult<WeiXinUserBo>()
                    .setExecuteResult(false)
                    .setResult(Result.Failure);
        }

        Agent agent = agentService.getAgent(agentId);
        if (agent == null) {
            LOG.info("未查找到商家信息, agentId:{}", agentId);
            return new ExecuteResult<WeiXinUserBo>()
                    .setExecuteResult(false)
                    .setResult(Result.Failure);
        }

        return internalOpenId(code, agent.getMchAppId(), agent.getMchAppSecret());
    }
}
