package com.demo.wjj.service.impl;

import com.demo.wjj.mapper.SaleMapper;
import com.demo.wjj.po.Sale;
import com.demo.wjj.service.SaleService;
import com.demo.wjj.utils.CommitTransactional;
import com.demo.wjj.utils.ReadOnlyTransactional;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 销售
 * @author wangqing
 * @date 2018.11.10
 */
@Service
public class SaleServiceImpl implements SaleService {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    @Autowired
    private SaleMapper saleMapper;

    @CommitTransactional
    @Override
    public boolean save(Sale sale) {
        LOG.info("进入保存销售员, sale:{}", sale);

        boolean saveResult = saleMapper.insert(sale) > 0;
        LOG.info("退出保存销售员, result => {}", saveResult);
        return saveResult;
    }

    @ReadOnlyTransactional
    @Override
    public Sale getSale(String agentId, String openId) {
        LOG.info("进入获取销售人员, agentId:{}, openId:{}", agentId, openId);

        if (StringUtils.isBlank(agentId)) {
            LOG.warn("商家id为空");
            return null;
        }

        if (StringUtils.isBlank(openId)) {
            LOG.warn("微信openId为空");
            return null;
        }

        Sale sale = saleMapper.selectSale(agentId, openId);
        LOG.info("退出获取销售人员, sale:{}", sale);
        return sale;
    }

    @ReadOnlyTransactional
    @Override
    public Sale getSale(String saleId) {
        LOG.info("进入获取销售员信息, saleId:{}", saleId);
        if (StringUtils.isBlank(saleId)) {
            LOG.info("saleId为空");
            return null;
        }
        Sale sale = saleMapper.selectSaleOfId(saleId);
        LOG.debug("查询的销售员信息:{}", sale);
        LOG.info("退出获取销售员信息");
        return sale;
    }

    @ReadOnlyTransactional
    @Override
    public boolean existsOpenId(String agentId, String openId) {
        LOG.info("z进入查询微信号是否已注册销售员, agentId:{}, openId:{}", agentId, openId);
        if (StringUtils.isBlank(agentId)) {
            LOG.warn("商家id为空");
            return false;
        }

        if (StringUtils.isBlank(openId)) {
            LOG.warn("openId为空");
            return false;
        }
        int count = saleMapper.selectExistsOpenId(agentId, openId);
        LOG.debug("微信号数量:{}, agentId:{}, openId:{}", count, agentId, openId);
        boolean exists = count > 0;
        LOG.info("退出查询微信号是否已注册销售员, exists:{}, openId:{}", exists, openId);
        return exists;
    }

    @ReadOnlyTransactional
    @Override
    public boolean existsMobile(String agentId, String mobile) {
        LOG.info("进入查询手机号是否已注册销售员, agentId:{}, mobile:{}", agentId, mobile);

        if (StringUtils.isBlank(agentId)) {
            LOG.warn("商家id为空");
            return false;
        }

        if (StringUtils.isBlank(mobile)) {
            LOG.warn("mobile为空");
            return false;
        }

        int count = saleMapper.selectExistsMobile(agentId, mobile);
        LOG.debug("手机号数量:{}, agentId:{}, mobile:{}", count, agentId, mobile);
        boolean exists = count > 0;
        LOG.info("退出查询手机号是否已注册销售员, exists:{}, mobile:{}", exists, mobile);
        return exists;
    }

    @CommitTransactional
    @Override
    public boolean unbind(String agentId, String openId) {
        LOG.info("进入解绑销售员, agentId:{}, openId:{}", agentId, openId);

        if (StringUtils.isBlank(agentId)) {
            LOG.info("agentId为空");
            return false;
        }

        if (StringUtils.isBlank(openId)) {
            LOG.info("openId为空");
            return false;
        }

        boolean updateResult = saleMapper.updateRegisterStatus(agentId, openId, Sale.REGISTER_STAUTS_UNBINDING) > 0;
        LOG.info("退出解绑销售员, result => {}", updateResult);
        return updateResult;
    }
    @Override
    public boolean updateOpenId(String agentId, String mobile, String openId) {
        return saleMapper.updateOpenId(agentId,mobile,openId)>0;
    }

    @Override
    public boolean updateMemberLevel(String agentId, String mobile, Integer memberLevel) {
        LOG.info("进入更新销售员会员等级, agentId:{}, mobile:{},memberLevel:{}", agentId, mobile,memberLevel);

        return saleMapper.updateMemberLevel(agentId, mobile, memberLevel)>0;
    }

    @Override
    public boolean addIntegral(String openId, Integer integral) {
        LOG.info("进入更新销售员会员积分, openId:{}, integral:{}", openId,integral);
        return saleMapper.updateIntegral(openId,integral)>0;
    }

    @Override
    public boolean updateMemberLevelBySaleId(String saleId, Integer memberLevel) {
        LOG.info("进入更新销售员会员等级, saleId:{}, memberLevel:{}", saleId,memberLevel);
        return saleMapper.updateMemberLevelBySaleId(saleId, memberLevel)>0;
    }

    @Override
    public boolean updatePlatTpSet(String saleId, Integer platTpSet) {
        LOG.info("进入更新销售员会员通票, saleId:{}, memberLevel:{}", saleId,platTpSet);
        return saleMapper.updatePlatTpSet(saleId, platTpSet)>0;
    }


}
