package com.demo.wjj.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.demo.wjj.service.TicketManage;
import com.demo.wjj.service.TokenManage;
import com.demo.wjj.utils.HttpResult;
import com.demo.wjj.utils.HttpUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * 票据管理
 * @author wangqing
 * @date 2018.12.01
 */
@Service
public class TicketManageImpl implements TicketManage {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    private final Map<String, Ticket> ticketCache = new HashMap<>();

    @Autowired
    private TokenManage tokenManage;

    @Override
    public String getTicket(String appId, String appSecret) {
        LOG.info("进入获取ticket, appId:{}, appSecret:{}", appId, appSecret);
        final String ticket = internalGetTicket(appId, appSecret);
        LOG.info("退出获取ticket, ticket:{}", ticket);
        return ticket;
    }

    /**
     * 获取ticket
     * @param appId appId
     * @param appSecret appSecret
     * @return ticket
     */
    private String internalGetTicket(String appId, String appSecret) {
        LOG.info("进入内部获取票据, appId:{}, appSecret:{}", appId, appSecret);
        final Ticket ticket = ticketCache.get(appId);
        if (ticket == null) {
            LOG.info("票据缓存中未查到票据");
        } else {
            final Date createTime = ticket.createTime;
            if (createTime == null) {
                LOG.info("票据缓存中的票据已过期, ticket:{}", ticket);
            } else {
                // 票据有效期（秒）
                final int ticketExpiry = 90 * 60;
                if ((System.currentTimeMillis() - createTime.getTime()) / 1000 > ticketExpiry) {
                    LOG.info("票据缓存中的票据已过期, ticket:{}", ticket);
                } else {
                    LOG.info("票据缓存中的票据, ticket:{}", ticket);
                    return ticket.getTicket();
                }
            }
        }
        Ticket newTicket = requestTicket(appId, appSecret);
        if (newTicket == null) {
            LOG.info("请求到的ticket为空");
            return null;
        }
        ticketCache.put(appId, newTicket);

        LOG.info("退出内部获取票据, ticket:{}", newTicket.getTicket());
        return newTicket.getTicket();
    }

    /**
     * 请求微信获取ticket
     * @param appId appId
     * @param appSecret appSecret
     * @return ticket
     */
    private Ticket requestTicket(String appId, String appSecret) {
        LOG.info("进入请求微信票据, appId:{}, appSecret:{}", appId, appSecret);

        /*
        final String accessToken = tokenManage.getAccessToken(appId, appSecret);
        LOG.info("获取的accessToken:{}, appId:{}", accessToken, appId);

        if (StringUtils.isBlank(accessToken)) {
            LOG.info("获取的accessToken为空");
            return null;
        }
         */

        final WeChatServiceImpl.PlatformAuthorization platformAuthorization = WeChatServiceImpl.PLATFORM_AUTHORIZATION_CACHE.get(appId);
        if (platformAuthorization == null) {
            LOG.info("未从缓存中查询到平台授权accessToken");
            return null;
        }

        final int tryTimes = 5;
        final Ticket ticket = internalRequestTicket(appId, appSecret, platformAuthorization.getAccessToken(), tryTimes);
        if (ticket == null) {return null;}
        LOG.info("退出请求微信票据, ticket:{}", ticket);
        return ticket;
    }

    /**
     * 请求ticket
     * @param appId appId
     * @param appSecret appSecret
     * @param accessToken accessToken
     * @param tryTimes 尝试次数
     * @return ticket
     */
    private Ticket internalRequestTicket(String appId, String appSecret, String accessToken, int tryTimes) {
        LOG.info("进入内部请求微信票据, appId:{}, appSecret:{}, accessToken:{}, tryTimes:{}", appId, appSecret, accessToken, tryTimes);
        final String apiUrlStr = "https://api.weixin.qq.com/cgi-bin/ticket/getticket?access_token=%s&type=jsapi";
        final String apiUrl = String.format(apiUrlStr, accessToken);

        final HttpResult httpResult = HttpUtils.post(apiUrl, new HashMap<String, Object>());
        if (httpResult.getStatusCode() != 200) {
            LOG.info("请求微信获取票据失败, statusCode:{}", httpResult.getStatusCode());
            return null;
        }

        final String content = httpResult.getContent();
        if (StringUtils.isBlank(content)) {
            LOG.info("请求微信获取票据响应内容为空");
            return null;
        }

        final JSONObject jsonObject = JSON.parseObject(content);
        final Integer errcode = jsonObject.getInteger("errcode");
        /*
        if (errcode != null && errcode == 40014 && tryTimes > 0) {
            return internalRequestTicket(appId, appSecret, tokenManage.requestAccessToken(appId, appSecret), tryTimes - 1);
        } else if (errcode != 0) {
            LOG.info("请求微信获取票据失败. errcode:{}", errcode);
            return null;
        }
         */
        if (errcode != 0) {
            LOG.info("请求微信获取票据失败. errcode:{}", errcode);
            return null;
        }

        final Ticket ticket = new Ticket();
        ticket.setTicket(jsonObject.getString("ticket"));
        ticket.setCreateTime(new Date());
        LOG.info("退出内部微信票据, ticket:{}", ticket);
        return ticket;
    }

    private static class Ticket {
        private String ticket;

        private Date createTime;

        @Override
        public String toString() {
            return JSON.toJSONString(this);
        }

        public String getTicket() {
            return ticket;
        }

        public void setTicket(String ticket) {
            this.ticket = ticket;
        }

        public Date getCreateTime() {
            return createTime;
        }

        public void setCreateTime(Date createTime) {
            this.createTime = createTime;
        }
    }
}
