package com.demo.wjj.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.demo.wjj.bo.AuthorizationUrlBo;
import com.demo.wjj.bo.CallbackTicketBo;
import com.demo.wjj.bo.DisplaceDetailBo;
import com.demo.wjj.mapper.NumMapper;
import com.demo.wjj.mapper.WeChatMapper;
import com.demo.wjj.mapper.WeiXinMessageMapper;
import com.demo.wjj.po.*;
import com.demo.wjj.service.*;
import com.demo.wjj.utils.CommitTransactional;
import com.demo.wjj.utils.HttpResult;
import com.demo.wjj.utils.HttpUtils;
import com.demo.wjj.utils.UuidUtils;
import com.demo.wjj.utils.wechat.AesException;
import com.demo.wjj.utils.wechat.WXBizMsgCrypt;
import org.apache.commons.lang3.StringUtils;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import sun.management.resources.agent;

import java.io.ByteArrayInputStream;
import java.nio.charset.StandardCharsets;
import java.util.*;

/**
 * 微信
 * @author wangqing
 * @date 2019.01.30
 */
@Service
public class WeChatServiceImpl implements WeChatService, InitializingBean {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    public static final Map<String, Token> TOKEN_CACHE = new HashMap<>();

    public static final Map<String, PlatformAuthorization> PLATFORM_AUTHORIZATION_CACHE = new HashMap<>();

    /**
     * 模板缓存
     * {
     *     "授权appId1": {
     *         "模板名称1": "模板id1",
     *         "模板名称2": "模板id2"
     *     }
     * }
     */
    public static final Map<String, Map<String, String>> TEMPLATE_CACHE = new HashMap<>();

    @Autowired
    private AgentService agentService;

    @Autowired
    private NumMapper numMapper;

    /**
     * 第三方平台appId
     */
    @Value("wx6af4ca6c083a5918")
    private String appId;

    /**
     * 第三方平台秘钥
     */
    @Value("515bb52094b522fd958bee043b42a9ec")
    private String appSecret;

    /**
     * 第三方平台自定义
     */
    @Value("BensF7xcrZBFnHOnSuAL")
    private String token;

    /**
     * 第三方平台自定义
     */
    @Value("${platform.encodingAESKeying}")
    private String encodingAESKeying;

    @Value("${weixin.pay.domain}")
    private String domain;

    @Value("0")
    private String templateId;

    @Value("${weixin.appId}")
    private String xiaoChengxuAppId;

    @Autowired
    private WeChatMapper weChatMapper;

    @Autowired
    private WeiXinTicketService weiXinTicketService;

    @Autowired
    private WeiXinPlatformAuthorizationService weiXinPlatformAuthorizationService;

    @Autowired
    private DiggerAgentDisplaceManageService diggerAgentDisplaceManageService;

    @CommitTransactional
    @Override
    public boolean parseCallbackTicket(CallbackTicketBo callbackTicketBo) throws AesException, DocumentException {
        LOG.info("进入解析微信第三方平台票据回调, callbackTicketBo:{}", callbackTicketBo);

        if (callbackTicketBo == null) {
            LOG.info("票据回调为空");
            return false;
        }

        WXBizMsgCrypt wxBizMsgCrypt = new WXBizMsgCrypt(token, encodingAESKeying, appId);
        // 解密后的xml
        final String decryptXml = wxBizMsgCrypt.decryptMsg(callbackTicketBo.getMsgSignature(), callbackTicketBo.getTimestamp(),
                callbackTicketBo.getNonce(), callbackTicketBo.getPostdata());
        LOG.debug("票据回调, 解密后内容:{}", decryptXml);
        ByteArrayInputStream inputStream = new ByteArrayInputStream(decryptXml.getBytes(StandardCharsets.UTF_8));

        SAXReader saxReader = new SAXReader();
        Document document = saxReader.read(inputStream);
        Element rootElement = document.getRootElement();

        final Element appIdElement = rootElement.element("AppId");
        if (appIdElement == null) {
            LOG.info("没有AppId节点元素");
            return false;
        }
        String appId = appIdElement.getStringValue();
        if (StringUtils.isBlank(appId)) {
            LOG.info("appId为空");
            return false;
        }

        final Element infoTypeElement = rootElement.element("InfoType");
        if (infoTypeElement == null) {
            LOG.info("没有InfoType节点元素");
            return false;
        }
        final String infoType = infoTypeElement.getStringValue();
        if (StringUtils.isBlank(infoType)) {
            LOG.info("infoType为空");
            return false;
        }
        final String infoTypeValue = "component_verify_ticket";
        if (!infoTypeValue.equals(infoType)) {
            LOG.info("infoType值非法, infoType:{}, 应为:{}", infoType, infoTypeValue);
            return true;
        }

        final Element ticketElement = rootElement.element("ComponentVerifyTicket");
        if (ticketElement == null) {
            LOG.info("没有ComponentVerifyTicket节点元素");
            return false;
        }
        final String ticket = ticketElement.getStringValue();
        if (StringUtils.isBlank(ticket)) {
            LOG.info("ticket为空");
            return false;
        }

        WeiXinTicket weiXinTicket = new WeiXinTicket(appId, ticket);
        if (!weiXinTicketService.save(weiXinTicket)) {
            LOG.info("保存微信第三方平台票据失败");
            throw new RuntimeException("保存微信第三方平台票据失败");
        }

        final Token token = TOKEN_CACHE.get(appId);
        if (token == null || token.isExpire()) {
            LOG.info("微信第三方平台token已失效");
            initPlatformToken(ticket);
        }
        LOG.info("退出解析微信第三方平台票据回调");
        return true;
    }

    @CommitTransactional
    @Override
    public boolean resolveCallbackAuthorization(String authorizationCode, String id, String authType) {
        LOG.info("进入解析第三方平台回调授权码, authorizationCode:{}, id:{}, authType:{}", authorizationCode, id, authType);

        if (StringUtils.isBlank(authorizationCode)) {
            LOG.info("授权码为空");
            return false;
        }

        if (StringUtils.isBlank(id)) {
            LOG.info("商家id为空");
            return false;
        }

        if (StringUtils.isBlank(authType)) {
            LOG.info("authTyp为空");
            return false;
        }

        final String token = getToken(appId);
        if (StringUtils.isBlank(token)) {
            LOG.info("token为空");
            return false;
        }
        // 使用授权码换取公众号或小程序的接口调用凭据和授权信息
        final String apiUrl = "https://api.weixin.qq.com/cgi-bin/component/api_query_auth?component_access_token=" + token;
        Map<String, Object> requestParameters = new HashMap<>();
        requestParameters.put("component_appid", appId);
        requestParameters.put("authorization_code", authorizationCode);
        final HttpResult httpResult = HttpUtils.post(apiUrl, JSON.toJSONString(requestParameters).getBytes(StandardCharsets.UTF_8));
        if (httpResult.getStatusCode() != 200) {
            LOG.info("请求公众号或小程序的接口调用凭证失败, statusCode:{}", httpResult.getStatusCode());
            return false;
        }

        final String content = httpResult.getContent();
        if (StringUtils.isBlank(content)) {
            LOG.info("请求公众号或小程序的接口调用凭证, 响应内容为空");
            return false;
        }
        LOG.debug("请求公众号或小程序的接口调用凭证, 响应内容:{}", content);
        try {
            final JSONObject jsonObject = JSON.parseObject(content);
            final JSONObject authorizationInfo = jsonObject.getJSONObject("authorization_info");
            if (authorizationInfo == null) {
                LOG.info("响应内容, 授权信息为空");
                return false;
            }

            final String authorizerAppid = authorizationInfo.getString("authorizer_appid");
            if (StringUtils.isBlank(authorizerAppid)) {
                LOG.info("授权方appId为空");
                return false;
            }

            final String authorizerAccessToken = authorizationInfo.getString("authorizer_access_token");
            if (StringUtils.isBlank(authorizerAccessToken)) {
                LOG.info("授权方接口调用访问令牌为空");
                return false;
            }

            final String authorizerRefreshToken = authorizationInfo.getString("authorizer_refresh_token");
            if (StringUtils.isBlank(authorizerRefreshToken)) {
                LOG.info("接口调用凭据刷新令牌为空");
                return false;
            }

            WeiXinPlatformAuthorization weiXinPlatformAuthorization = new WeiXinPlatformAuthorization();
            weiXinPlatformAuthorization.setAppId(appId);
            weiXinPlatformAuthorization.setAuthorizationAppId(authorizerAppid);
            weiXinPlatformAuthorization.setAccessToken(authorizerAccessToken);
            weiXinPlatformAuthorization.setRefreshToken(authorizerRefreshToken);
            weiXinPlatformAuthorization.setJsonStr(content);
            Date now = new Date();
            weiXinPlatformAuthorization.setCreateTime(now);
            weiXinPlatformAuthorization.setModifyTime(now);
            if (!weiXinPlatformAuthorizationService.save(weiXinPlatformAuthorization)) {
                LOG.info("保存微信第三方平台授权方信息失败");
                throw new RuntimeException("保存微信第三方平台授权方信息失败");
            }

            final PlatformAuthorization platformAuthorization = new PlatformAuthorization(appId, authorizerAppid, authorizerAccessToken, authorizerRefreshToken);
            LOG.info("微信第三方平台授权方accessToken放入缓存, authorizarAppId:{}, platformAuthorization:{}", authorizerAppid, platformAuthorization);
            PLATFORM_AUTHORIZATION_CACHE.put(authorizerAppid, platformAuthorization);
            updateInfo(id, authorizerAppid, authType);
            LOG.info("退出解析第三方平台回调授权码");
            return true;
        } catch (Exception e) {
            LOG.error("解析公众号或小程序的接口调用凭证, 响应内容异常", e);
            return false;
        }
    }

    private boolean updateInfo(String id, String authorizerAppId, String authType) {
        LOG.info("进入更新商家信息, id:{}, authorizerAppId:{}, authType:{}", id, authorizerAppId, authType);
        if ("1".equals(authType)) {
            final String apiUrl = "https://api.weixin.qq.com/cgi-bin/component/api_get_authorizer_info?component_access_token=" + getToken(appId);
            Map<String, String> requestParameters = new HashMap<>();
            requestParameters.put("component_appid", appId);
            requestParameters.put("authorizer_appid", authorizerAppId);

            final HttpResult httpResult = HttpUtils.post(apiUrl, JSON.toJSONString(requestParameters).getBytes(StandardCharsets.UTF_8));
            if (httpResult.getStatusCode() != 200) {
                LOG.info("请求微信公众号基础信息失败, statusCode:{}", httpResult.getStatusCode());
                return false;
            }

            final String content = httpResult.getContent();
            if (StringUtils.isBlank(content)) {
                LOG.info("请求微信公众号基础信息接口,响应内容为空");
                return false;
            }

            LOG.info("请求微信公众号基础信息接口, 响应内容:{}", content);

            String name = null;
            try {
                final JSONObject jsonObject = JSON.parseObject(content);
                final JSONObject info = jsonObject.getJSONObject("authorizer_info");
                name = info.getString("nick_name");
            } catch (Exception e) {
                LOG.error("解析微信公众号基础信息响应内容异常", e);
            }

            final boolean result = agentService.updateAgentInfo(id, authorizerAppId, name);
            if (!result) {
                LOG.info("更新商家信息失败");
                return false;
            }
            return true;
        } else {
            final boolean result = agentService.updateXcxAppId(id, authorizerAppId);
            if (!result) {
                LOG.info("更新商家小程序appId失败");
                return false;
            }
            return true;
        }
    }

    private void initPlatformToken(String ticket) {
        LOG.info("进入初始化第三方平台token, ticket:{}", ticket);

        final String apiUrl = "https://api.weixin.qq.com/cgi-bin/component/api_component_token";

        Map<String, Object> requestParameters = new HashMap<>();
        requestParameters.put("component_appid", appId);
        requestParameters.put("component_appsecret", appSecret);
        requestParameters.put("component_verify_ticket", ticket);
        final HttpResult httpResult = HttpUtils.post(apiUrl, JSON.toJSONString(requestParameters).getBytes(StandardCharsets.UTF_8));

        if (httpResult.getStatusCode() != 200) {
            LOG.info("请求获取第三方平台token失败, statusCode:{}", httpResult.getStatusCode());
            return;
        }

        final String content = httpResult.getContent();
        if (StringUtils.isBlank(content)) {
            LOG.info("请求获取第三方平台token, 响应内容为空");
            return;
        }
        LOG.debug("请求获取第三方平台token, 响应内容:{}", content);

        try {
            final JSONObject jsonObject = JSON.parseObject(content);
            final String token = jsonObject.getString("component_access_token");
            if (StringUtils.isBlank(token)) {
                LOG.info("token内容为空");
                return;
            }
            LOG.debug("微信第三方平台token放入缓存, appId:{}, ticket:{}, token:{}", appId, ticket, token);
            TOKEN_CACHE.put(appId, new Token(ticket, token, new Date()));
        } catch (Exception e) {
            LOG.error("解析第三方平台token响应内容异常", e);
            return;
        }

        LOG.info("退出初始化第三方平台token");
    }

    @Override
    public void afterPropertiesSet() {
        new Thread(new Runnable() {
            @Override
            public void run() {
                LOG.info("启动初始化微信第三方票据线程开始");
                final List<WeiXinTicket> ticketList = weiXinTicketService.getTicketList();
                if (CollectionUtils.isEmpty(ticketList)) {
                    LOG.info("微信第三方票据为空");
                    return;
                }

                for (WeiXinTicket weiXinTicket : ticketList) {
                    if (StringUtils.isBlank(weiXinTicket.getTicket())) {
                        LOG.info("ticket为空");
                        continue;
                    }
                    initPlatformToken(weiXinTicket.getTicket());
                }
                LOG.info("启动初始化微信第三方票据线程结束");
            }
        }).start();

        new Thread(new Runnable() {
            final int sleepLength = 1 * 60 * 1000;
            @Override
            public void run() {
                LOG.info("启动监听微信第三方票据过期线程");
                while (true) {
                    if (!TOKEN_CACHE.isEmpty()) {
                        final Collection<Token> tokens = TOKEN_CACHE.values();
                        for (Token t : tokens) {
                            if (t.isExpire()) {
                                initPlatformToken(t.getTicket());
                            }
                        }
                    }
                    try {
                        Thread.sleep(sleepLength);
                    } catch (InterruptedException e) {
                        LOG.error("暂停线程异常", e);
                    }
                }
            }
        }).start();

        new Thread(new Runnable() {
            @Override
            public void run() {
                LOG.info("启动初始化授权方accessToken线程开始");
                List<WeiXinPlatformAuthorization> weiXinPlatformAuthorizations = weiXinPlatformAuthorizationService.getPlatformAuthorization();
                if (weiXinPlatformAuthorizations.isEmpty()) {
                    LOG.info("授权方accessToken集合为空");
                    return;
                }

                for (WeiXinPlatformAuthorization weiXinPlatformAuthorization : weiXinPlatformAuthorizations) {
                    final PlatformAuthorization platformAuthorization = new PlatformAuthorization(weiXinPlatformAuthorization.getAppId(), weiXinPlatformAuthorization.getAuthorizationAppId(),
                            weiXinPlatformAuthorization.getAccessToken(), weiXinPlatformAuthorization.getRefreshToken(), weiXinPlatformAuthorization.getCreateTime());
                    PLATFORM_AUTHORIZATION_CACHE.put(platformAuthorization.getAuthorizationAppId(), platformAuthorization);
                }
            }
        }).start();

        new Thread(new Runnable() {
            final int sleepLength = 1 * 60 * 1000;
            @Override
            public void run() {
                LOG.info("启动监听授权方accessToken过期线程开始");
                while (true) {
                    if (PLATFORM_AUTHORIZATION_CACHE.isEmpty()) {
                        LOG.info("授权方accessToken缓存为空");
                    } else {
                        final Collection<PlatformAuthorization> platformAuthorizations = PLATFORM_AUTHORIZATION_CACHE.values();
                        for (PlatformAuthorization platformAuthorization : platformAuthorizations) {
                            if (platformAuthorization.isExpire()) {
                                refreshAccessToken(platformAuthorization);
                            }
                        }
                    }
                    try {
                        Thread.sleep(sleepLength);
                    } catch (InterruptedException e) {
                        LOG.error("暂停线程异常", e);
                    }
                }
            }
        }).start();
    }

    private void refreshAccessToken(PlatformAuthorization platformAuthorization) {
        LOG.info("进入刷新授权方accessToken, platformAuthorization:{}", platformAuthorization);

        if (platformAuthorization == null) {
            LOG.info("授权方信息为空");
            return;
        }

        final String token = getToken(platformAuthorization.getAppId());
        if (StringUtils.isBlank(token)) {
            LOG.info("token为空");
            return;
        }
        final String apiUrl = "https://api.weixin.qq.com/cgi-bin/component/api_authorizer_token?component_access_token=" + token;
        Map<String, Object> requestParameters = new HashMap<>();
        requestParameters.put("component_appid", platformAuthorization.getAppId());
        requestParameters.put("authorizer_appid", platformAuthorization.getAuthorizationAppId());
        requestParameters.put("authorizer_refresh_token", platformAuthorization.getRefreshToken());

        final Date now = new Date();

        final HttpResult httpResult = HttpUtils.post(apiUrl, JSON.toJSONString(requestParameters).getBytes(StandardCharsets.UTF_8));
        if (httpResult.getStatusCode() != 200) {
            LOG.info("请求微信刷新授权方accessToken失败");
            return;
        }

        final String content = httpResult.getContent();
        if (StringUtils.isBlank(content)) {
            LOG.info("请求微信刷新授权方accessToken, 响应内容为空");
            return;
        }

        try {
            final JSONObject jsonObject = JSON.parseObject(content);
            final String accessToken = jsonObject.getString("authorizer_access_token");
            final String refreshToken = jsonObject.getString("authorizer_refresh_token");
            if (StringUtils.isBlank(accessToken)) {
                LOG.info("accessToken为空");
                return;
            }

            if (StringUtils.isBlank(refreshToken)) {
                LOG.info("refreshToken为空");
                return;
            }

            if (!weiXinPlatformAuthorizationService.updateAccessTokenAndRefreshToken(platformAuthorization.getAuthorizationAppId(), accessToken, refreshToken)) {
                LOG.info("更新授权方accessToken和refreshToken失败");
                return;
            }
            LOG.info("刷新微信第三方平台授权方accessToken放入缓存, authorizarAppId:{}, platformAuthorization:{}", platformAuthorization.getAuthorizationAppId(), platformAuthorization);
            PLATFORM_AUTHORIZATION_CACHE.put(platformAuthorization.getAuthorizationAppId(), platformAuthorization.build(accessToken, refreshToken, now));

        } catch (Exception e) {
            LOG.error("解析微信刷新授权方accessToken响应内容异常", e);
        }
    }

    @Override
    public String getToken(String appId) {
        final Token token = TOKEN_CACHE.get(appId);
        if (token == null) {
            return null;
        }
        return token.getContent();
    }

    public static class Token {
        private String ticket;
        private String content;
        private Date createTime;

        public Token(String ticket, String content, Date createTime) {
            setTicket(ticket);
            setContent(content);
            setCreateTime(createTime);
        }

        public boolean isExpire() {
            final Calendar expireTime = Calendar.getInstance();
            final Calendar now = Calendar.getInstance();
            expireTime.setTime(createTime);
            expireTime.add(Calendar.HOUR_OF_DAY, 1);
            expireTime.add(Calendar.MINUTE, 30);
            return expireTime.getTime().before(now.getTime());
        }

        public void setTicket(String ticket) {
            if (StringUtils.isBlank(ticket)) {
                throw new IllegalArgumentException("ticket为空");
            }
            this.ticket = ticket;
        }

        public String getTicket() {
            return ticket;
        }

        public void setContent(String content) {
            if (StringUtils.isBlank(content)) {
                throw new IllegalArgumentException("content为空");
            }
            this.content = content;
        }

        public void setCreateTime(Date createTime) {
            if (createTime == null) {
                throw new IllegalArgumentException("createTime为空");
            }
            this.createTime = createTime;
        }

        public String getContent() {
            return content;
        }
    }

    public class PlatformAuthorization {

        /**
         * 第三方应用appId
         */
        private String appId;

        /**
         * 授权方appId
         */
        private String authorizationAppId;

        private String accessToken;

        private String refreshToken;

        private Date createTime;

        public PlatformAuthorization build(String accessToken, String refreshToken, Date createTime) {
            return new PlatformAuthorization(this.appId, this.authorizationAppId, accessToken, refreshToken, createTime);
        }

        public PlatformAuthorization(String appId, String authorizationAppId, String accessToken, String refreshToken, Date createTime) {
            this.appId = appId;
            this.authorizationAppId = authorizationAppId;
            this.accessToken = accessToken;
            this.refreshToken = refreshToken;
            this.createTime = createTime;
        }

        public PlatformAuthorization(String appId, String authorizationAppId, String accessToken, String refreshToken) {
            this(appId, authorizationAppId, accessToken, refreshToken, new Date());
        }

        public boolean isExpire() {
            Calendar expireDate = Calendar.getInstance();
            Calendar now = Calendar.getInstance();
            expireDate.setTime(createTime);
            expireDate.add(Calendar.HOUR_OF_DAY, 1);
            expireDate.add(Calendar.MINUTE, 30);
            return expireDate.getTime().before(now.getTime());
        }

        @Override
        public String toString() {
            return JSON.toJSONString(this);
        }

        public String getAppId() {
            return appId;
        }

        public void setAppId(String appId) {
            this.appId = appId;
        }

        public String getAuthorizationAppId() {
            return authorizationAppId;
        }

        public void setAuthorizationAppId(String authorizationAppId) {
            this.authorizationAppId = authorizationAppId;
        }

        public String getAccessToken() {
            return accessToken;
        }

        public void setAccessToken(String accessToken) {
            this.accessToken = accessToken;
        }

        public String getRefreshToken() {
            return refreshToken;
        }

        public void setRefreshToken(String refreshToken) {
            this.refreshToken = refreshToken;
        }

        public Date getCreateTime() {
            return createTime;
        }

        public void setCreateTime(Date createTime) {
            this.createTime = createTime;
        }
    }

    @Override
    public String generateAuthorizationUrl(AuthorizationUrlBo authorizationUrlBo) {
        LOG.info("进入生成第三方平台授权url, authorizationUrlBo:{}", authorizationUrlBo);
        final String callbackUrl;
        // 公众号
        if ("1".equals(authorizationUrlBo.getAuthType())) {
            callbackUrl = "https://www.xxinjiyuan.com/wjj-api/wechat/authorization/code/callback?id={id}";
        } else {
            // 小程序
            callbackUrl = "https://www.xxinjiyuan.com/wjj-api/wechat/authorization/code/xcx/callback?id={id}";
        }

        final String authorizationUrlStr = "https://mp.weixin.qq.com/safe/bindcomponent?action=bindcomponent&auth_type=${authType}&no_scan=1&component_appid=${appId}&pre_auth_code=${preAuthCode}&redirect_uri=${callback}&biz_appid=${bizAppId}#wechat_redirect";

        final String bizAppId = StringUtils.defaultIfBlank(authorizationUrlBo.getBizAppId(), "");

        final String authorizationUrl = authorizationUrlStr.replace("${authType}", authorizationUrlBo.getAuthType())
                .replace("${appId}", appId)
                .replace("${preAuthCode}", getPreAuthCode(appId))
//                .replace("${preAuthCode}", "1")
                .replace("${callback}", callbackUrl)
                .replace("${bizAppId}", bizAppId);

        LOG.info("退出生成第三方平台授权url, result:{}", authorizationUrl);
        return authorizationUrl;
    }

    @Override
    public String getMenus(String authorizerAppid) {
        LOG.info("进入获取公众号菜单, authorizerAppId:{}", authorizerAppid);

        if (StringUtils.isBlank(authorizerAppid)) {
            LOG.info("授权方appId为空");
            return null;
        }

        if (!PLATFORM_AUTHORIZATION_CACHE.containsKey(authorizerAppid)) {
            LOG.info("未查找到授权方appId对应的accessToken. authorizationAppId:{}", authorizerAppid);
            return null;
        }

        final String apiUrl = "https://api.weixin.qq.com/cgi-bin/menu/get?access_token=" +
                PLATFORM_AUTHORIZATION_CACHE.get(authorizerAppid).getAccessToken();

        final HttpResult httpResult = HttpUtils.post(apiUrl, new HashMap<String, Object>());
        if (httpResult.getStatusCode() != 200) {
            LOG.info("请求微信公众号菜单接口失败, statusCode:{}", httpResult.getStatusCode());
            return null;
        }

        final String content = httpResult.getContent();
        if (StringUtils.isBlank(content)) {
            LOG.info("请求微信公众号菜单接口, 响应内容为空");
            return null;
        }
        LOG.info("退出获取公众号菜单, result:{}", content);
        return content;
    }

    @Override
    public String deleteMenus(String authorizerAppid) {
        LOG.info("进入删除公众号菜单, authorizerAppid:{}", authorizerAppid);

        if (StringUtils.isBlank(authorizerAppid)) {
            LOG.info("授权方appId为空");
            return null;
        }

        if (!PLATFORM_AUTHORIZATION_CACHE.containsKey(authorizerAppid)) {
            LOG.info("未查找到授权方appId对应的accessToken. authorizationAppId:{}", authorizerAppid);
            return null;
        }

        final String apiUrl = "https://api.weixin.qq.com/cgi-bin/menu/delete?access_token=" +
                PLATFORM_AUTHORIZATION_CACHE.get(authorizerAppid).getAccessToken();

        final HttpResult httpResult = HttpUtils.post(apiUrl, new HashMap<String, Object>());
        if (httpResult.getStatusCode() != 200) {
            LOG.info("请求删除公众号菜单接口失败, statusCode:{}", httpResult.getStatusCode());
            return null;
        }

        final String content = httpResult.getContent();
        if (StringUtils.isBlank(content)) {
            LOG.info("请求删除公众号菜单接口, 响应内容为空");
            return null;
        }

        LOG.info("退出删除公众号菜单, content:{}", content);
        return content;
    }

    @Override
    public String createMenus(String authorizerAppid, String menusJson) {
        LOG.info("进入创建公众号菜单, authorizerAppid:{}, menusJson:{}", authorizerAppid, menusJson);

        if (StringUtils.isBlank(authorizerAppid)) {
            LOG.info("授权方appId为空");
            return null;
        }

        if (StringUtils.isBlank(menusJson)) {
            LOG.info("公众号菜单menusJson为空");
            return null;
        }

        if (!PLATFORM_AUTHORIZATION_CACHE.containsKey(authorizerAppid)) {
            LOG.info("为查找到授权方appId对应的accessToken. authorizationAppId:{}", authorizerAppid);
            return null;
        }

        final String apiUrl = "https://api.weixin.qq.com/cgi-bin/menu/create?access_token=" +
                PLATFORM_AUTHORIZATION_CACHE.get(authorizerAppid).getAccessToken();

        final HttpResult httpResult = HttpUtils.post(apiUrl, menusJson.getBytes(StandardCharsets.UTF_8));
        if (httpResult.getStatusCode() != 200) {
            LOG.info("请求创建公众号菜单接口失败, statusCode:{}", httpResult.getStatusCode());
            return null;
        }

        final String content = httpResult.getContent();
        if (StringUtils.isBlank(content)) {
            LOG.info("请求创建公众号菜单接口, 响应内容为空");
            return null;
        }

        return content;
    }

    @Override
    public Map<String, String> getTemplate(String id, String templateName) {
        LOG.info("进入获取微信模板, id:{}, templateName:{}", id, templateName);
        /*
        *
        *
        * *///商家
        final Agent agent;
        //如果长度小于13 是agentId
        if (id.length()<=13){
             agent = agentService.getAgent(id);
        }else {
             agent = agentService.getAgentOfAppId(id);
        }
        if (agent == null) {
            LOG.info("未查找到商家信息g, id:{}", id);
            return null;
        }

        final PlatformAuthorization platformAuthorization = PLATFORM_AUTHORIZATION_CACHE.get(agent.getAppId());
        if (platformAuthorization == null) {
            LOG.info("未在缓存中找到授权方accessToken信息");
            return null;
        }

        String _appId = agent.getAppId();

        if (TEMPLATE_CACHE.containsKey(_appId)) {
            final Map<String, String> templateCache = TEMPLATE_CACHE.get(_appId);
            LOG.info("从缓存中读取模板id, {}", templateCache);
            final String templateId = templateCache.get(templateName);
            if (StringUtils.isNoneBlank(templateId)) {
                Map<String, String> result = new HashMap<>();
                result.put("templateId", templateId);

                result.put("accessToken", platformAuthorization.getAccessToken());
                LOG.info("退出获取微信模板, result:{}", result);
                return result;
            }
        }

        String publicTemplateId;
        if ("报价申请反馈通知".equals(templateName)) {
            publicTemplateId = "OPENTM401424226";
        } else if ("出价结束通知".equals(templateName)) {
            publicTemplateId = "OPENTM408985532";
        } else if ("审核结果通知".equals(templateName)) {
            publicTemplateId = "OPENTM405456700";
        } else if ("拍品上架通知".equals(templateName)) {
            publicTemplateId = "OPENTM207294943";
        } else if ("等待审核通知".equals(templateName)) {
            publicTemplateId = "OPENTM408471635";
        } else if("闲置上架成功通知".equals(templateName)) {
            publicTemplateId = "OPENTM400181196";
        } else if("作品审核通知".equals(templateName)) {
            publicTemplateId = "OPENTM410241803";
        }else if("客服处理提醒".equals(templateName)) {
            publicTemplateId = "OPENTM202172872";
        }else if("商家回复提醒".equals(templateName)) {
            publicTemplateId = "OPENTM401003457";
        }else if("下载成功提示".equals(templateName)) {
            publicTemplateId = "OPENTM207867722";
        }else if("实名认证审核结果通知".equals(templateName)) {
            publicTemplateId = "OPENTM417959522";
        }else if("认证审核提醒".equals(templateName)) {
            publicTemplateId = "OPENTM415235881";
        }else if("订单待确认通知".equals(templateName)) {
            publicTemplateId = "OPENTM416102658";
        } else {
            LOG.info("请求微信模板列表, 模板名称错误。templateName:{}", templateName);
            return null;
        }

        /*
         *删除模板（每调用20次执行一次）
         */
        //查询执行次数类
        TbNum tbNum = numMapper.findObjectById(1);
        if (tbNum.getStep().equals(tbNum.getSize())){
            if (clearTemplate(platformAuthorization)) {
                LOG.info("微信模板列表 清除失败");
                return null;
            }
            //重新计数
            tbNum.setStep(0);
            int update = numMapper.update(tbNum);
            if (update==1){
                LOG.info("计数归零成功", tbNum);
            }else {
                LOG.info("计数归零失败", tbNum);
            }
        }else {
            //步长加一
            tbNum.setStep(tbNum.getStep()+1);
            numMapper.update(tbNum);
        }



        LOG.info("请求微信公用模板id, publicTemplateId:{}", publicTemplateId);
        Map<String, Object> p1 = new HashMap<>();
        p1.put("template_id_short", publicTemplateId);
        final HttpResult post = HttpUtils.post("https://api.weixin.qq.com/cgi-bin/template/api_add_template?access_token=" + platformAuthorization.getAccessToken(), JSON.toJSONString(p1).getBytes(StandardCharsets.UTF_8));
        if (post.getStatusCode() != 200) {
            LOG.info("请求微信公用模板id失败, statusCode:{}", post.getStatusCode());
            return null;
        }
        final String c1 = post.getContent();
        LOG.info("请求微信公用模板id，响应内容:{}", c1);
        final JSONObject jo1 = JSON.parseObject(c1);
        final Integer errcode1 = jo1.getInteger("errcode");
        if (errcode1== null || errcode1 != 0) {
            LOG.info("请求微信公用模板id失败。");
            return null;
        }

        final String templateId1 = jo1.getString("template_id");
        Map<String, String> result = new HashMap<>();
        if (templateId1 != null && !"".equals(templateId1)) {
            result.put("templateId", templateId1);
            result.put("accessToken", platformAuthorization.getAccessToken());
        }

        if (TEMPLATE_CACHE.containsKey(_appId)) {
            TEMPLATE_CACHE.get(_appId).put(templateName, templateId1);
        } else {
            Map<String, String> cache = new HashMap<>();
            cache.put(templateName, templateId1);
            TEMPLATE_CACHE.put(_appId,cache);
        }

        /*
        final String apiUrlStr = "https://api.weixin.qq.com/cgi-bin/template/get_all_private_template?access_token=%s";

        final String apiUrl = String.format(apiUrlStr, platformAuthorization.getAccessToken());
        HttpResult httpResult = HttpUtils.post(apiUrl, new HashMap<String, Object>());

        if (httpResult.getStatusCode() != 200) {
            LOG.info("请求微信模板列表失败, statusCode:{}", httpResult.getStatusCode());
            return null;
        }

        String content = httpResult.getContent();
        if (StringUtils.isBlank(content)) {
            LOG.info("请求微信模板列表, 响应内容为空");
            return null;
        }

        JSONObject jsonObject = JSON.parseObject(content);

        final Integer errcode = jsonObject.getInteger("errcode");

        JSONArray templateList = jsonObject.getJSONArray("template_list");
        if (org.apache.commons.collections4.CollectionUtils.isEmpty(templateList)) {
            LOG.info("未查询到微信消息模板, appId:{}", appId);
            return null;
        }

        Map<String, String> result = new HashMap<>();
        Map<String, String> cache = new HashMap<>();
        JSONObject each;
        for (int i = 0, step = templateList.size(); i < step; i++) {
            each = templateList.getJSONObject(i);
            String title = each.getString("title");
            final String templateId = each.getString("template_id");
            cache.put(title, templateId);
            if (templateName.equals(title)) {
                LOG.info("退出获取置换审核模板id, templateId:{}", each.getString("template_id"));
                result.put("templateId", templateId);
                result.put("accessToken", platformAuthorization.getAccessToken());
            }
        }
        if (!cache.isEmpty()) {
            TEMPLATE_CACHE.put(appId, cache);
        }
         */
            LOG.info("退出获取微信模板, result:{}", result);
            return result;
        }

    /**
     * 删除微信多余模板
      * @param platformAuthorization
     * @return
     */
    private boolean clearTemplate(PlatformAuthorization platformAuthorization) {

        //获取微信模板列表
        final HttpResult list = HttpUtils.get("https://api.weixin.qq.com/cgi-bin/template/get_all_private_template?access_token=" + platformAuthorization.getAccessToken());
        if (list.getStatusCode() != 200) {
            LOG.info("请求微信模板类表失败, statusCode:{}", list.getStatusCode());
            return true;
        }
        String content = list.getContent();
        if (StringUtils.isBlank(content)) {
            LOG.info("请求微信模板列表, 响应内容为空");
            return true;
        }
        LOG.info("开始解析");
        JSONObject jsonObject = JSON.parseObject(content);
        JSONArray templateList = jsonObject.getJSONArray("template_list");
        LOG.info("获取的模板集合：",templateList);
        //创建存放模板id 和 模板名称的map集合
        HashMap<String,String> map = new HashMap();
        //取出模板id 和 模板名称 放入map中
        JSONObject each;
        for (int i = 0; i < templateList.size(); i++) {
            each = templateList.getJSONObject(i);
            final String template_id = each.getString("template_id");
            final String title = each.getString("title");
            map.put(template_id,title);
        }
        LOG.info("现有的微信模板：",map);
        //创建一个集合存放值，遍历map集合获取所有的模板名称
        ArrayList<String> valuesList=new ArrayList<>();
        //获取map的key--模板id
        Set<String> template_id=map.keySet();
        //遍历每个key（模板id） 获取对应的值--模板名称
        for (String key : template_id) {
        valuesList.add(map.get(key));
    }
        System.out.println("=========================="+valuesList);
        //遍历list集合，向后遍历找到相同的元素(模板名称)，添加到新的listrepeat集合中
        ArrayList<String> listrepeat=new ArrayList<>();
        for (int i = 0; i < valuesList.size(); i++) {
            for (int j = i+1; j < valuesList.size(); j++) {
                if (valuesList.get(i).equals(valuesList.get(j))) {
                    listrepeat.add(valuesList.get(i));
                }
            }
        }
        //遍历map集合，如果值包含在listrepeat中，则返回键的值，存入集合restkey中
        ArrayList<String> restTemplateId=new ArrayList<>();
        for (String key : template_id) {
            if (listrepeat.contains(map.get(key))) {
                restTemplateId.add(key);
            }
        }
        System.out.println("======================"+restTemplateId);
        //删除掉在集合restkey中出现的键的元素
        for (String templateId : restTemplateId) {
//            map.remove(string);
            //调用微信删除模板接口删除重复id
            Map<String, Object> p1 = new HashMap<>();
            p1.put("template_id", templateId);
            final HttpResult post = HttpUtils.post("https://api.weixin.qq.com/cgi-bin/template/del_private_template?access_token=" + platformAuthorization.getAccessToken(), JSON.toJSONString(p1).getBytes(StandardCharsets.UTF_8));
            if (post.getStatusCode() != 200) {
                LOG.info("微信模板id删除失败, statusCode:{}", post.getStatusCode());
                return true;
            }
            final String c1 = post.getContent();
            LOG.info("删除微信模板id，响应内容:{}", c1);

            final JSONObject jo1 = JSON.parseObject(c1);
            final Integer errcode1 = jo1.getInteger("errcode");
            if (errcode1== null || errcode1 != 0) {
                LOG.info("微信模板id删除失败。");
                return true;
            }
            LOG.info("微信模板id删除成功。");
        }
        return false;
    }




    @Override
    public String uploadCode(String id) {
        LOG.info("进入上传小程序代码, id:{}", id);

        final Agent agent = agentService.getAgentOfId(id);
        if (agent == null) {
            LOG.info("未查询到agent. id:{}", id);
            return "";
        }

        String authorizerAppid = agent.getXcxAppId();

        final PlatformAuthorization platformAuthorization = PLATFORM_AUTHORIZATION_CACHE.get(authorizerAppid);
        if (platformAuthorization == null) {
            LOG.info("未在缓存中找到授权方accessToken信息");
            return "";
        }
        String token = platformAuthorization.getAccessToken();
        /*
        final String token = getToken(appId);
        if (StringUtils.isBlank(token)) {
            LOG.info("获取第三方平台token失败");
            return "";
        }
        */

        // https://open.weixin.qq.com/cgi-bin/showdocument?action=dir_list&t=resource/res_list&verify=1&id=open1489138143_WPbOO&token=&lang=zh_CN

        final String modifyDomainUrl = "https://api.weixin.qq.com/wxa/modify_domain?access_token=" + token;
        Map<String, Object> params = new HashMap<>();
        params.put("action", "set");
        params.put("requestdomain", new String[]{"https://www.xxinjiyuan.com"});
        params.put("wsrequestdomain", new String[]{"wss://www.xxinjiyuan.com"});
        params.put("uploaddomain", new String[]{"https://www.xxinjiyuan.com"});
        params.put("downloaddomain", new String[]{"https://www.xxinjiyuan.com"});
        final HttpResult httpResult1 = HttpUtils.post(modifyDomainUrl, JSON.toJSONString(params).getBytes(StandardCharsets.UTF_8));
        if (httpResult1.getStatusCode() != 200) {
            LOG.info("请求修改小程序服务器域名失败");
            return "{\"errcode\":-1, \"errmsg\":\"请求修改小程序服务器域名失败\"}";
        }

        final String content1 = httpResult1.getContent();
        if (StringUtils.isBlank(content1)) {
            LOG.info("请求修改小程序服务器域名，响应内容为空");
            return "{\"errcode\":-1, \"errmsg\":\"请求修改小程序服务器域名失败\"}";
        }
        LOG.info("请求修改小程序服务器域名，响应内容:{}", content1);
        final JSONObject jsonObject1 = JSON.parseObject(content1);
        final Integer errcode = jsonObject1.getInteger("errcode");
        if (errcode != 0 && errcode != 85017) {
            LOG.info("请求修改小程序服务器域名失败");
            return content1;
        }

        final String modifyBussinessDomainUrl = "https://api.weixin.qq.com/wxa/setwebviewdomain?access_token=" + token;
        Map<String, Object> params2 = new HashMap<>();
        params2.put("action", "set");
        params2.put("webviewdomain", new String[]{"https://www.xxinjiyuan.com"});
        final HttpResult httpResult2 = HttpUtils.post(modifyBussinessDomainUrl, JSON.toJSONString(params2).getBytes(StandardCharsets.UTF_8));
        if (httpResult2.getStatusCode() != 200) {
            LOG.info("请求修改小程序业务域名失败");
            return "{\"errcode\":-1, \"errmsg\":\"请求修改小程序业务域名失败\"}";
        }

        final String content2 = httpResult2.getContent();
        if (StringUtils.isBlank(content2)) {
            LOG.info("请求修改小程序业务域名，响应内容为空");
            return "{\"errcode\":-1, \"errmsg\":\"请求修改小程序业务域名失败\"}";
        }

        LOG.info("请求修改小程序业务域名，响应内容:{}", content2);
        final JSONObject jsonObject2 = JSON.parseObject(content2);
        final Integer errcode2 = jsonObject2.getInteger("errcode");
        if (errcode2 != 0 && errcode2 != 89019) {
            LOG.info("请求修改小程序业务域名失败");
            return content2;
        }


        final String apiUrl = "https://api.weixin.qq.com/wxa/commit?access_token=" + token;

        Map<String, String> requestParameters = new HashMap<>();
        requestParameters.put("template_id", templateId);

        JSONObject jsonObject = new JSONObject();
        jsonObject.put("extEnable", true);
        jsonObject.put("extAppid", authorizerAppid);
        JSONObject ext = new JSONObject();
//        ext.put("agentId", agent.getAgentId());
        ext.put("agentId", agent.getAgentId());
        ext.put("appid", authorizerAppid);
        ext.put("appId", authorizerAppid);
        jsonObject.put("ext", ext);

        JSONArray pages = new JSONArray();
        pages.add("index");
//        jsonObject.put("pages", pages);

        requestParameters.put("ext_json", jsonObject.toJSONString());
        requestParameters.put("user_version", "v1.0");
        requestParameters.put("user_desc", "test");

        LOG.info("调用微信上传小程序代码接口, 请求json:{}", JSON.toJSONString(requestParameters));
        final HttpResult httpResult = HttpUtils.post(apiUrl, JSON.toJSONString(requestParameters).getBytes(StandardCharsets.UTF_8));
        if (httpResult.getStatusCode() != 200) {
            LOG.info("调用微信上传小程序代码接口失败.statusCode:{}", httpResult.getStatusCode());
            return "";
        }

        final String content = httpResult.getContent();
        if (StringUtils.isBlank(content)) {
            LOG.info("调用微信上传小程序代码接口，响应内容为空");
            return "";
        }

        LOG.info("退出上传小程序代码, result:{}", content);
        return content;
    }

    @Override
    public String getCategory(String id) {
        LOG.info("进入获取微信小程序已设置的类目, id:{}", id);

        final Agent agent = agentService.getAgentOfId(id);
        if (agent == null) {
            LOG.info("未查询到商家信息, id:{}", id);
            return "";
        }

        String authorizerAppid = agent.getXcxAppId();
        final PlatformAuthorization platformAuthorization = PLATFORM_AUTHORIZATION_CACHE.get(authorizerAppid);
        if (platformAuthorization == null) {
            LOG.info("未在缓存中找到授权方accessToken信息");
            return "";
        }

        final String apiUrl = "https://api.weixin.qq.com/wxa/get_category?access_token=" + platformAuthorization.getAccessToken();
        final HttpResult httpResult = HttpUtils.get(apiUrl);
        if (httpResult.getStatusCode() != 200) {
            LOG.info("调用获取微信小程序已设置的类目失败, statusCode:{}", httpResult.getStatusCode());
            return "";
        }

        final String content = httpResult.getContent();
        if (StringUtils.isBlank(content)) {
            LOG.info("调用获取微信小程序已设置的类目，响应内容为空");
            return "";
        }

        LOG.info("退出获取微信小程序已设置的类目, result:{}", content);
        return content;
    }

    public static void main(String[] args) {
        String token = "19_dje-ysKSvpRNeDWlltcJUj4VA5yiasuiRz7TyRbwpOeQXnoJ5L9YCNkzL-bl3VJt-uFgRZVPbKr4-ngFnVVU53NDo0tkI3_qkwiEeGhH9C2njW01yyed2L-9w6yIW7OLVqTj_f_NhoH-1iN_WURhAIAAYV";
        final String apiUrl = "https://api.weixin.qq.com/cgi-bin/component/api_get_authorizer_list?component_access_token=" + token;
        Map<String, Object> params = new HashMap<>();
        params.put("component_appid", "wx6af4ca6c083a5918");
        params.put("offset", 0);
        params.put("count", 500);
        final HttpResult post = HttpUtils.post(apiUrl, JSON.toJSONString(params).getBytes(StandardCharsets.UTF_8));
        System.out.println(post.getContent());
    }

    @Override
    public String getPages(String id) {
        LOG.info("进入获取微信小程序已设置的页面, id:{}", id);

        final Agent agent = agentService.getAgentOfId(id);
        if (agent == null) {
            LOG.info("未查询到商家信息, id:{}", id);
            return "";
        }

        String authorizerAppid = agent.getXcxAppId();
        final PlatformAuthorization platformAuthorization = PLATFORM_AUTHORIZATION_CACHE.get(authorizerAppid);
        if (platformAuthorization == null) {
            LOG.info("未在缓存中找到授权方accessToken信息");
            return "";
        }

        final String apiUrl = "https://api.weixin.qq.com/wxa/get_page?access_token=" + platformAuthorization.getAccessToken();
        final HttpResult httpResult = HttpUtils.get(apiUrl);
        if (httpResult.getStatusCode() != 200) {
            LOG.info("调用获取微信小程序已设置的页面失败, statusCode:{}", httpResult.getStatusCode());
            return "";
        }

        final String content = httpResult.getContent();
        if (StringUtils.isBlank(content)) {
            LOG.info("调用获取微信小程序已设置的页面，响应内容为空");
            return "";
        }

        LOG.info("退出获取微信小程序已设置的页面, result:{}", content);
        return content;
    }

    @Override
    public String submitAudit(String id, String json) {
        LOG.info("进入提交小程序审核, authorizerAppid:{}, json:{}", json);

        final Agent agent = agentService.getAgentOfId(id);
        if (agent == null) {
            LOG.info("未查询到商家信息, id:{}", id);
            return "";
        }

        final PlatformAuthorization platformAuthorization = PLATFORM_AUTHORIZATION_CACHE.get(agent.getXcxAppId());
        if (platformAuthorization == null) {
            LOG.info("未在缓存中找到授权方accessToken信息");
            return "";
        }

        final String apiUrl = "https://api.weixin.qq.com/wxa/submit_audit?access_token=" + platformAuthorization.getAccessToken();
        final HttpResult httpResult = HttpUtils.post(apiUrl, json.getBytes(StandardCharsets.UTF_8));
        if (httpResult.getStatusCode() != 200) {
            LOG.info("调用提交小程序审核接口失败, statusCode:{}", httpResult.getStatusCode());
            return "";
        }

        final String content = httpResult.getContent();
        if (StringUtils.isBlank(content)) {
            LOG.info("调用提交小程序审核接口，响应内容为空");
            return "";
        }

        LOG.info("退出提交小程序审核, result:{}", content);
        return content;
    }

    @Override
    public String publishCode(String id) {
        LOG.info("进入发布小程序, id:{}", id);

        final Agent agent = agentService.getAgentOfId(id);
        if (agent == null) {
            LOG.info("未查询到商家信息, id:{}", id);
            return "";
        }

        final PlatformAuthorization platformAuthorization = PLATFORM_AUTHORIZATION_CACHE.get(agent.getXcxAppId());
        if (platformAuthorization == null) {
            LOG.info("未在缓存中找到授权方accessToken信息");
            return "";
        }

        final String apiUrl = "https://api.weixin.qq.com/wxa/release?access_token=" + platformAuthorization.getAccessToken();
        final HttpResult httpResult = HttpUtils.post(apiUrl, "{}".getBytes(StandardCharsets.UTF_8));
        if (httpResult.getStatusCode() != 200) {
            LOG.info("调用发布小程序接口失败, statusCode:{}", httpResult.getStatusCode());
            return "";
        }

        final String content = httpResult.getContent();
        if (StringUtils.isBlank(content)) {
            LOG.info("调用发布小程序接口失败，响应内容为空");
            return "";
        }

        LOG.info("退出发布小程序, result:{}", content);
        return content;
    }

    @Override
    public boolean pushWaitShelvesMessage(String id, String displaceId) {
        LOG.info("调用推送待上架微信消息, id:{}, displaceId:{}", id, displaceId);

        if (StringUtils.isBlank(id)) {
            LOG.info("置换id为空");
            return false;
        }

        if (StringUtils.isBlank(displaceId)) {
            LOG.info("置换id为空");
            return false;
        }

        final Agent agent = weChatMapper.selectAgentByDisplaceId(id);
        if (agent == null) {
            LOG.info("未查询到商家信息, displaceId:{}", id);
            return false;
        }

        if ("1".equalsIgnoreCase(agent.getAgentType())) {
            LOG.info("商家为自运营类型，不用推送待上架消息");
            return true;
        }

        final List<Map<String, Object>> userList = weChatMapper.selectPushMessageUsers(agent.getAgentId());
        if (userList == null || userList.isEmpty()) {
            LOG.info("未查询到该商家的管理员信息. agentId:{}", agent.getAgentId());
            return false;
        }

        final Map<String, String> template = getTemplate("XJY00", "等待上架通知");
        if (template == null) {
            LOG.info("未查询到 等待上架通知 消息模板");
            return false;
        }

        final Map<String, Object> sale = weChatMapper.selectSale(id);
        final String saleName = StringUtils.defaultIfBlank((String) sale.get("sale_name"), "");

        final DisplaceDetailBo displaceDetailBo = diggerAgentDisplaceManageService.queryDetails(id);
        if (displaceDetailBo == null) {
            LOG.info("未查询到置换设备信息, id:{}", id);
            return false;
        }
        String deviceName = displaceDetailBo.getBrand() + " " + displaceDetailBo.getModel() + " " + displaceDetailBo.getDeviceType();

        final String templateId = template.get("templateId");
        boolean result = false;
        for (Map<String, Object> user : userList) {
            final String wxh = (String) user.get("user_wxh");
            if (StringUtils.isBlank(wxh)) {
                LOG.info("用户微信号为空, id:{}", user.get("id"));
            }

            // 推送给微工程用户
            final boolean pushResult = _pushMessage(templateId, PLATFORM_AUTHORIZATION_CACHE.get("wx0cb0cbc34027621f").getAccessToken(), wxh, saleName, deviceName,
                    agent, displaceId, id, (String) user.get("user_phone"));

            if (pushResult && !result) {
                result = true;
            }

        }
        return result;
    }

    @Value("${displaceAuditHost}")
    private String displaceAuditUrl;

    @Autowired
    private WeiXinMessageMapper weiXinMessageMapper;

    private boolean _pushMessage(String ti, String accessToken, String wxh, String saleName, String deviceName, Agent agent, String displaceId, String id, String mobile) {
        LOG.info("进入推送待上架微信消息, templateId:{}, accessToken:{}, wxh:{}, saleName:{}, deviceName:{}, agent:{}, displaceId:{}, id:{}, mobile:{}",
                ti, accessToken, wxh, saleName, deviceName, agent, displaceId, id, mobile);
        final String apiUrlStr = "https://api.weixin.qq.com/cgi-bin/message/template/send?access_token=%s";
        final String apiUrl = String.format(apiUrlStr, accessToken);

        Map<String, Object> parameters = new HashMap<>();

        parameters.put("touser", wxh);
        parameters.put("template_id", ti);
        parameters.put("url", String.format("%s/wx/wxApplogin/%s.mp", displaceAuditUrl, "XJY00"));

        Map<String, Map<String, String>> data = new HashMap<>();
        final Map<String, String> first = new HashMap<>();
        first.put("color", "#173177");
        first.put("value", "有一条置换消息待上架");
        data.put("first", first);

        final Map<String, String> keyword1 = new HashMap<>();
        keyword1.put("color", "#173177");
        keyword1.put("value", saleName);
        data.put("keyword1",keyword1);

        final Map<String, String> keyword2 = new HashMap<>();
        keyword2.put("color", "#173177");
        keyword2.put("value", deviceName);
        data.put("keyword2", keyword2);

        final Map<String, String> remark = new HashMap<>();
        remark.put("color", "#173177");
        remark.put("value", "点击【详情】前往审核");
        data.put("remark", remark);
        parameters.put("data", data);

        final HttpResult httpResult = HttpUtils.post(apiUrl, JSON.toJSONString(parameters).getBytes(StandardCharsets.UTF_8));

        if (httpResult.getStatusCode() != 200) {
            LOG.info("请求微信发送待上架消息失败, statusCode:{}", httpResult.getStatusCode());
            return false;
        }

        final String content = httpResult.getContent();
        if (StringUtils.isBlank(content)) {
            LOG.info("请求微信发送待上架消息, 响应内容为空");
            savePushRecord(agent, displaceId, id, wxh, saleName, mobile, "0");
            return false;
        }

        final JSONObject jsonObject = JSON.parseObject(content);
        final Integer errcode = jsonObject.getInteger("errcode");
        if (errcode != null && errcode == 42001 ) {
            LOG.info("请求微信发送待上架消息失败, accessToken已过期.");
            savePushRecord(agent, displaceId, id, wxh, saleName, mobile, "0");
            return false;
        } else if (errcode != 0) {
            LOG.info("请求微信发送待上架消息失败, 推送失败. errcode:{}", errcode);
            savePushRecord(agent, displaceId, id, wxh, saleName, mobile, "0");
            return false;
        }

        savePushRecord(agent, displaceId, id, wxh, saleName, mobile, "1");
        return true;
    }

    private boolean savePushRecord(Agent agent, String displaceId, String wxh, String wxnc, String id, String mobile, String pushStatus) {
        PushRecord pushRecord = new PushRecord();
        pushRecord.setId(UuidUtils.generate());
        pushRecord.setAgentId(agent.getAgentId());
        pushRecord.setAgentName(agent.getAgentName());
        pushRecord.setDisplaceId(displaceId);
        pushRecord.setMenuName("zhsh");
        pushRecord.setMsgType("sbsc");
        pushRecord.setReceiveWxh(wxh);
        pushRecord.setReceiveWxnc(wxnc);
        pushRecord.setPushTime(new Date());
        pushRecord.setPushStatus(pushStatus);
        pushRecord.setReferId(id);
        pushRecord.setPushType("1");
        pushRecord.setReceiveName(wxh);
        pushRecord.setReceivePhone(mobile);
        return weiXinMessageMapper.insertPushRecord(pushRecord) > 0;
    }

    private String getPreAuthCode(String appId) {
        LOG.info("进入请求预授权码, appId:{}", appId);

        final String token = getToken(appId);
        if (StringUtils.isBlank(token)) {
            LOG.info("获取第三方平台token失败");
            return null;
        }

        final String apiUrl = "https://api.weixin.qq.com/cgi-bin/component/api_create_preauthcode?component_access_token=" + token;
        Map<String, Object> requestParameters = new HashMap<>();
        requestParameters.put("component_appid", appId);
        final HttpResult httpResult = HttpUtils.post(apiUrl, JSON.toJSONString(requestParameters).getBytes(StandardCharsets.UTF_8));
        if (httpResult.getStatusCode() != 200) {
            LOG.info("请求预授权码失败, statusCode:{}", httpResult.getStatusCode());
            return null;
        }

        final String content = httpResult.getContent();
        if (StringUtils.isBlank(content)) {
            LOG.info("请求预授权码, 响应内容为空");
            return null;
        }
        LOG.debug("请求预授权码, 响应内容:{}", content);

        try {
            final JSONObject jsonObject = JSON.parseObject(content);
            final String preAuthCode = jsonObject.getString("pre_auth_code");
            if (StringUtils.isBlank(preAuthCode)) {
                LOG.info("预授权码为空");
                return null;
            }
            LOG.info("退出请求预授权码, result:{}", preAuthCode);
            return preAuthCode;
        } catch (Exception e) {
            LOG.error("解析预授权码响应内容异常", e);
            return null;
        }
    }


}
