package com.demo.wjj.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.demo.wjj.bo.SaveDisplaceBo;
import com.demo.wjj.mapper.WeiXinMessageMapper;
import com.demo.wjj.po.Agent;
import com.demo.wjj.po.PushRecord;
import com.demo.wjj.po.Sale;
import com.demo.wjj.po.User;
import com.demo.wjj.service.TokenManage;
import com.demo.wjj.service.WeChatService;
import com.demo.wjj.service.WeiXinMessageService;
import com.demo.wjj.utils.CommitTransactional;
import com.demo.wjj.utils.HttpResult;
import com.demo.wjj.utils.HttpUtils;
import com.demo.wjj.utils.UuidUtils;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.nio.charset.StandardCharsets;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 微信消息推送
 * @author wangqing
 * @date 2018.11.23
 */
@Service
public class WeiXinMessageServiceImpl implements WeiXinMessageService {

    private final Logger LOG = LoggerFactory.getLogger(getClass());

    @Value("${displaceAuditHost}")
    private String displaceAuditUrl;

    @Value("${displaceAuditTemplateName}")
    private String displaceAuditTemplateName;

    @Autowired
    private WeiXinMessageMapper weiXinMessageMapper;

    @Autowired
    private WeChatService weChatService;

    @Autowired
    private TokenManage tokenManage;

    @CommitTransactional
    @Override
    public boolean send(Agent agent, Sale sale, SaveDisplaceBo saveDisplaceBo) {
        LOG.info("进入微信推送置换审核消息, agent:{}, sale:{}, saveDisplaceBo:{}", agent, sale, saveDisplaceBo);


        final WeChatServiceImpl.PlatformAuthorization platformAuthorization = WeChatServiceImpl.PLATFORM_AUTHORIZATION_CACHE.get(agent.getAppId());
        if (platformAuthorization == null) {
            LOG.info("从缓存中未找到授权方信息, appId:{}", agent.getAppId());
            return false;
        }

        String accessToken = platformAuthorization.getAccessToken();

        if (StringUtils.isBlank(accessToken)) {
            LOG.info("获取accessToken失败");
            return false;
        }

        final int tryTimes = 5;
        final Map<String, String> template = weChatService.getTemplate(agent.getAgentId(), displaceAuditTemplateName);
        if (template == null) {
            LOG.info("未查询到置换审核模板id");
            return false;
        }
        final String templateId = template.get("templateId");
//        final String templateId = getTemplateId(accessToken, agent.getAppId(), agent.getAppSecret(), tryTimes);
        if (StringUtils.isBlank(templateId)) {
            LOG.info("未查询到置换审核模板id");
            return false;
        }

        final boolean sendResult = sendMessage(accessToken, templateId, agent, sale, saveDisplaceBo, tryTimes);


        LOG.info("退出微信推送置换审核消息, result=> {}", sendResult);
        return sendResult;
    }

    /**
     * 推送微信置换审核消息
     * @param accessToken accessToken
     * @param templateId 模板id
     * @param agent 商家
     * @param sale 销售员
     * @param saveDisplaceBo 保存置换
     * @param tryTimes
     * @return 推送是否成功rereq
     */
    private boolean sendMessage(String accessToken, String templateId, Agent agent, Sale sale, SaveDisplaceBo saveDisplaceBo, int tryTimes) {
        LOG.info("进入推送置换审核微信消息, accessToken:{}, templateId:{}, agent:{}, sale:{}, saveDisplaceBo:{}", accessToken, templateId, agent, sale, saveDisplaceBo);

        final List<User> users = weiXinMessageMapper.selectAgentOpenId(agent.getAgentId());
        if (CollectionUtils.isEmpty(users)) {
            LOG.info("未查询到商家对应审核人员");
            return false;
        }

        for (User user : users) {
            internalSend(accessToken, templateId, agent, sale, saveDisplaceBo, user, tryTimes);
        }

        LOG.info("退出推送置换审核微信消息");
        return true;
    }

    private Boolean internalSend(String accessToken, String templateId, Agent agent, Sale sale, SaveDisplaceBo saveDisplaceBo, User user, int tryTimes) {
        LOG.info("进入微信消息推送, accessToken:{}, templateId:{}, agent:{}, sale:{}, saveDisplaceBo:{}, user:{}, tryTimes:{}", accessToken, templateId, agent, sale, saveDisplaceBo, user, tryTimes);
        final String apiUrlStr = "https://api.weixin.qq.com/cgi-bin/message/template/send?access_token=%s";
        final String apiUrl = String.format(apiUrlStr, accessToken);

        Map<String, Object> parameters = new HashMap<>();

        parameters.put("touser", user.getUserWx());
        parameters.put("template_id", templateId);
        parameters.put("url", String.format("%s/wx/wxApplogin/%s.mp", displaceAuditUrl, agent.getAgentId()));

        Map<String, Map<String, String>> data = new HashMap<>();
        final Map<String, String> first = new HashMap<>();
        first.put("color", "#173177");
        first.put("value", "您有一条新的置换信息未审核");
        data.put("first", first);

        final Map<String, String> keyword1 = new HashMap<>();
        keyword1.put("color", "#173177");
        keyword1.put("value", StringUtils.isBlank(sale.getSaleWxnc()) ? "" : sale.getSaleWxnc());
        data.put("keyword1",keyword1);

        final Map<String, String> keyword2 = new HashMap<>();
        keyword2.put("color", "#173177");
        keyword2.put("value", String.format("%s %s %s", saveDisplaceBo.getBrand(), saveDisplaceBo.getModel(), saveDisplaceBo.getDeviceType()));
        data.put("keyword2", keyword2);

        final Map<String, String> remark = new HashMap<>();
        remark.put("color", "#173177");
        remark.put("value", "请立即审核");
        data.put("remark", remark);
        parameters.put("data", data);

        final HttpResult httpResult = HttpUtils.post(apiUrl, JSON.toJSONString(parameters).getBytes(StandardCharsets.UTF_8));

        if (httpResult.getStatusCode() != 200) {
            LOG.info("请求微信发送置换审核消息失败, statusCode:{}", httpResult.getStatusCode());
            return false;
        }

        final String content = httpResult.getContent();
        if (StringUtils.isBlank(content)) {
            LOG.info("请求微信发送置换审核消息, 响应内容为空");
            return false;
        }

        final JSONObject jsonObject = JSON.parseObject(content);
        final Integer errcode = jsonObject.getInteger("errcode");
        if (errcode != null && errcode == 42001 && tryTimes > 0) {
            LOG.info("推送置换审核微信消息失败, accessToken已过期. 重新获取accessToken. tryTimes:{}", tryTimes);
            return false;
            //return sendMessage(tokenManage.requestAccessToken(agent.getAppId(), agent.getAppSecret()), templateId, agent, sale, saveDisplaceBo, tryTimes - 1);
        } else if (errcode != 0) {
            LOG.info("请求微信发送置换审核消息, 推送失败. errcode:{}", errcode);
            return false;
        }

        savePushRecord(agent, saveDisplaceBo, user);
        LOG.info("退出微信消息推送");
        return true;
    }

    private void savePushRecord(Agent agent, SaveDisplaceBo saveDisplaceBo, User user) {
        PushRecord pushRecord = new PushRecord();
        pushRecord.setId(UuidUtils.generate());
        pushRecord.setAgentId(agent.getAgentId());
        pushRecord.setAgentName(agent.getAgentName());
        pushRecord.setDisplaceId(saveDisplaceBo.getDisplaceId());
        pushRecord.setMenuName("zhsh");
        pushRecord.setMsgType("sbsc");
        pushRecord.setReceiveWxh(user.getUserWx());
        pushRecord.setReceiveWxnc(user.getUserWxnc());
        pushRecord.setPushTime(new Date());
        pushRecord.setPushStatus("1");
        pushRecord.setReferId(saveDisplaceBo.getDisplaceId());
        pushRecord.setPushType("1");
        pushRecord.setReceiveName(user.getName());
        pushRecord.setReceivePhone(user.getPhone());
        weiXinMessageMapper.insertPushRecord(pushRecord);
    }

    /**
     * 获取模板id
     * @param accessToken accessToken
     * @param appId
     * @param appSecret
     * @return 模板id
     */
    private String getTemplateId(String accessToken, String appId, String appSecret, int tryTimes) {
        LOG.info("进入获取置换审核模板id, accessToken:{}, appId: {}, appSecret: {}, tryTimes:{}", accessToken, appId, appSecret, tryTimes);

        final String apiUrlStr = "https://api.weixin.qq.com/cgi-bin/template/get_all_private_template?access_token=%s";

        final String apiUrl = String.format(apiUrlStr, accessToken);
        HttpResult httpResult = HttpUtils.post(apiUrl, new HashMap<String, Object>());

        if (httpResult.getStatusCode() != 200) {
            LOG.info("请求微信模板列表失败, statusCode:{}", httpResult.getStatusCode());
            return null;
        }

        String content = httpResult.getContent();
        if (StringUtils.isBlank(content)) {
            LOG.info("请求微信模板列表, 响应内容为空");
            return null;
        }

        JSONObject jsonObject = JSON.parseObject(content);

        final Integer errcode = jsonObject.getInteger("errcode");

        if (errcode != null && errcode == 42001 && tryTimes > 0) {
            LOG.info("请求微信获取模板接口, accessToken已过期. 重新获取accessToken");
            return null;
//            return getTemplateId(tokenManage.requestAccessToken(appId, appSecret), appId, appSecret, tryTimes - 1);
        }

        JSONArray templateList = jsonObject.getJSONArray("template_list");
        if (CollectionUtils.isEmpty(templateList)) {
            LOG.info("未查询到微信消息模板, appId:{}", appId);
            return null;
        }

        JSONObject each;
        for (int i = 0, step = templateList.size(); i < step; i++) {
             each = templateList.getJSONObject(i);

            String title = each.getString("title");
            if (displaceAuditTemplateName.equals(title)) {
                LOG.info("退出获取置换审核模板id, templateId:{}", each.getString("template_id"));
                return each.getString("template_id");
            }
        }

        LOG.info("退出获取置换审核模板id, 未查询到模板id");
        return null;
    }
}
